#define VARIANT_01
   // Classes and Objects

// #define VARIANT_02
   // Using Properties

// #define VARIANT_03
   // Static Properties

using System;

#if VARIANT_01
class Time
{
    private int hour;
    private int minute;
    private int second;

    public Time () : this (0, 0, 0)
    {
    }

    public Time (int hour, int minute, int second)
    {
        this.SetHour (hour);
        this.SetMinute (minute);
        this.SetSecond (second);
    }

    public void SetHour (int hour)
    {
        if (hour >= 0 && hour < 24)
        {
            this.hour = hour;
        }
    }

    public int GetHour ()
    {
        return this.hour;
    }

    public void SetMinute (int minute)
    {
        if (minute >= 0 && minute < 60)
        {
            this.minute = minute;
        }
    }

    public int GetMinute ()
    {
        return this.minute;
    }

    public void SetSecond (int second)
    {
        if (second >= 0 && second < 60)
        {
            this.second = second;
        }
    }

    public int GetSecond ()
    {
        return this.second;
    }

    public void Advance (int hours, int minutes, int seconds)
    {
        this.hour += hours;
        this.minute += minutes;
        this.second += seconds;
        this.Normalize ();
    }

    public void Reset (int hour, int minute, int second)
    {
        this.SetHour (hour);
        this.SetMinute (minute);
        this.SetSecond (second);
        this.Normalize ();
    }

    public bool PriorTo (Time t)
    {
        if (this.hour < t.hour)
            return true;
        else if (this.hour == t.hour)
        {
            if (this.minute < t.minute)
                return true;
            else if (this.minute == t.minute)
            {
                if (this.second < t.second)
                    return true;
            }
        }

        return false;
    }

    private void Normalize ()
    {
        this.minute += this.second / 60;
        this.hour += this.minute / 60;
        this.hour = this.hour % 24;
        this.minute = this.hour % 60;
        this.second = this.hour % 60;
    }

    public void Print ()
    {
        Console.Write ("{0:00}:{1:00}:{2:00}",
            this.hour, this.minute, this.second);
    }
}

class Demo
{
    public static void Main ()
    {
        Time t1 = new Time ();
        t1.SetHour (10);
        t1.SetMinute (11);
        t1.SetSecond (12);
        t1.Print ();
        Console.WriteLine ();

        Time t2 = new Time (23, 59, 59);
        t2.Print ();
        Console.WriteLine ();
        t2.Advance (0, 0, 1);
        t2.Print ();
        Console.WriteLine ();

        Time t3 = new Time (12, 0, 30);
        Time t4 = new Time (12, 0, 31);
        if (t3.PriorTo (t4))
        {
            t3.Print ();
            Console.Write (" is prior to ");
            t4.Print ();
        }
    }
}
#endif

#if VARIANT_02
class Time
{
    private int hour;
    private int minute;
    private int second;

    public Time () : this (0, 0, 0)
    {
    }

    public Time (int hour, int minute, int second)
    {
        this.Hour = hour;
        this.Minute = minute;
        this.Second = second;
    }

    public int Hour
    {
        set
        {
            if (value >= 0 && value < 24)
            {
                this.hour = value;
            }
        }

        get
        {
            return this.hour;
        }
    }

    public int Minute
    {
        set
        {
            if (value >= 0 && value < 60)
            {
                this.minute = value;
            }
        }

        get
        {
            return this.minute;
        }
    }

    public int Second
    {
        set
        {
            if (value >= 0 && value < 60)
            {
                this.second = value;
            }
        }

        get
        {
            return this.second;
        }
    }

    public void Advance (int hours, int minutes, int seconds)
    {
        this.hour += hours;
        this.minute += minutes;
        this.second += seconds;
        this.Normalize ();
    }

    public void Reset (int hour, int minute, int second)
    {
        this.Hour = hour;
        this.Minute = minute;
        this.Second = second;
        this.Normalize ();
    }

    public bool PriorTo (Time t)
    {
        if (this.hour < t.hour)
            return true;
        else if (this.hour == t.hour)
        {
            if (this.minute < t.minute)
                return true;
            else if (this.minute == t.minute)
            {
                if (this.second < t.second)
                    return true;
            }
        }

        return false;
    }

    private void Normalize ()
    {
        this.minute += this.second / 60;
        this.hour += this.minute / 60;
        this.hour = this.hour % 24;
        this.minute = this.hour % 60;
        this.second = this.hour % 60;
    }

    public override String ToString ()
    {
        String s = String.Format ("{0:00}:{1:00}:{2:00}",
            this.hour, this.minute, this.second);
        return s;
    }
}

class Demo
{
    public static void Main ()
    {
        Time t1 = new Time ();
        t1.Hour = 10;
        t1.Minute = 11;
        t1.Second = 12;
        Console.WriteLine (t1);

        Time t2 = new Time (23, 59, 59);
        Console.WriteLine (t2);
        t2.Advance (0, 0, 1);
        Console.WriteLine (t2);

        Time t3 = new Time (12, 0, 30);
        Time t4 = new Time (12, 0, 31);
        bool b = t3.PriorTo (t4);
        if (b)
            Console.WriteLine ("{0} is prior to {1}", t3, t4);
    }
}
#endif

#if VARIANT_03
class Time
{
    private static int instances;

    public static int Instances
    {
        get
        {
            return Time.instances;
        }
    }

    static Time ()
    {
        Time.instances = 0;
    }

    public Time ()
    {
        Time.instances ++;
    }
}

class Demo
{
    public static void Main ()
    {
        Time t1 = new Time ();
        Time t2 = new Time ();
        Time t3 = new Time ();

        Console.WriteLine ("Number of Time instances: {0}", Time.Instances);
    }
}
#endif

