using System;

enum BitArrayPosition { First, Last };

class BitArray
{
    private int bits;

    public BitArray ()
    {
        this.bits = 0;
    }

    public void SetBit (int index, bool value)
    {
        if (value)
        {
            int mask = 1 << index;
            this.bits |= mask;
        }
        else
        {
            int mask = ~(1 << index);
            this.bits &= mask;
        }
    }

    public bool GetBit (int index)
    {
        int n = this.bits & (1 << index);
        return (n != 0) ? true : false;
    }

    public int this [BitArrayPosition pos]
    {
        get
        {
            if (pos == BitArrayPosition.First)
            {
                for (int i = 0; i < 32; i ++)
                    if ((this.bits & (1 << i)) != 0)
                        return i;
            }
            else
            {
                for (int i = 31; i >= 0; i --)
                    if ((this.bits & (1 << i)) != 0)
                        return i;
            }

            return -1;
        }
    }

    public bool this [int index]
    {
        get
        {
            int n = this.bits & (1 << index);
            return (n != 0) ? true : false;
        }

        set
        {
            if (value)
            {
                int mask = 1 << index;
                this.bits |= mask;
            }
            else
            {
                int mask = ~(1 << index);
                this.bits &= mask;
            }
        }
    }

    public void Print ()
    {
        Console.WriteLine (this.bits);
    }
}

class Demo
{
    public static void Demo01 ()
    {
        BitArray ba = new BitArray ();

        ba[2] = true;
        ba[3] = true;
        ba[4] = true;
        ba.Print ();

        ba[3] = false;
        ba.Print ();

        bool b = ba[0];

        if (ba[4])
            Console.WriteLine ("bit is set");

        Console.WriteLine (b);
        b = ba[1];
        Console.WriteLine (b);
        b = ba[2];
        Console.WriteLine (b);
        b = ba[3];
        Console.WriteLine (b);
        b = ba[4];
        Console.WriteLine (b);
    }

    public static void Demo02 ()
    {
        BitArray ba = new BitArray ();
        ba[11] = true;
        ba[22] = true;

        int first = ba[BitArrayPosition.First];
        int last = ba[BitArrayPosition.Last];
        Console.WriteLine ("First: {0}, Last: {1}", first, last);

        ba[11] = false;
        ba[22] = false;
        first = ba[BitArrayPosition.First];
        last = ba[BitArrayPosition.Last];
        Console.WriteLine ("First: {0}, Last: {1}", first, last);
    }
    
    public static void Main ()
    {
        Demo01 ();
    }
}