using System;

class RationalNumber
{
    private int numerator;    // dividend
    private int denominator;  // divisor

    // c'tors
    public RationalNumber (int numerator, int denominator)
    {
        this.numerator = numerator;
        this.denominator = denominator;
    }

    public RationalNumber (int numerator) : this (numerator, 1)
    {
    }

    public RationalNumber () : this (1)
    {
    }

    // unary arithmetic operators
    public static RationalNumber operator+ (RationalNumber r)
    {
        return new RationalNumber (r.numerator, r.denominator);
    }

    public static RationalNumber operator- (RationalNumber r)
    {
        return new RationalNumber (-r.numerator, r.denominator);
    }

    public static RationalNumber operator++ (RationalNumber r)
    {
        int numerator = r.numerator + r.denominator;
        int denominator = r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator-- (RationalNumber r)
    {
        int numerator = r.numerator - r.denominator;
        int denominator = r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    // binary arithmetic operators
    public static RationalNumber operator+ (RationalNumber r1, RationalNumber r2)
    {
        int numerator =
            r1.numerator * r2.denominator + r1.denominator * r2.numerator;
        int denominator = r1.denominator * r2.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator+ (RationalNumber r, int n)
    {
        int numerator = r.numerator + n * r.denominator;
        int denominator = r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator+ (int n, RationalNumber r)
    {
        return r + n;
    }

    public static RationalNumber operator- (RationalNumber r1, RationalNumber r2)
    {
        int numerator =
            r1.numerator * r2.denominator - r1.denominator * r2.numerator;
        int denominator = r1.denominator * r2.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator- (RationalNumber r, int n)
    {
        int numerator = r.numerator - n * r.denominator;
        int denominator = r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator- (int n, RationalNumber r)
    {
        return r - n;
    }

    public static RationalNumber operator* (RationalNumber r1, RationalNumber r2)
    {
        int numerator = r1.numerator * r2.numerator;
        int denominator = r1.denominator * r2.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator* (RationalNumber r, int n)
    {
        int numerator = n * r.numerator;
        int denominator = r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator* (int n, RationalNumber r)
    {
        return r * n;
    }

    public static RationalNumber operator/ (RationalNumber r1, RationalNumber r2)
    {
        int numerator = r1.numerator * r2.denominator;
        int denominator = r1.denominator * r2.numerator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator/ (RationalNumber r, int n)
    {
        int numerator = r.numerator;
        int denominator = n * r.denominator;
        return new RationalNumber (numerator, denominator);
    }

    public static RationalNumber operator/ (int n, RationalNumber r)
    {
        int numerator = n * r.denominator;
        int denominator = r.numerator;
        return new RationalNumber (numerator, denominator);
    }

    // comparison operators
    public static bool operator== (RationalNumber r1, RationalNumber r2)
    {
        int n = r1.numerator * r2.denominator;
        int m = r1.denominator * r2.numerator;
        return (n == m) ? true : false;
    }

    public static bool operator!= (RationalNumber r1, RationalNumber r2)
    {
        return ! (r1 == r2);
    }

    public static bool operator< (RationalNumber r1, RationalNumber r2)
    {
        return
            (r1.numerator * r2.denominator ) < (r2.numerator * r1.denominator);
    }

    public static bool operator> (RationalNumber r1, RationalNumber r2)
    {
        return !((r1 < r2) || (r1 == r2));
    }

    public static bool operator<= (RationalNumber r1, RationalNumber r2)
    {
        return !(r1 > r2);
    }

    public static bool operator>= (RationalNumber r1, RationalNumber r2)
    {
        return !(r1 < r2);
    }
    
    // conversion operators
    public static explicit operator int (RationalNumber r)
    {
        float temp = (float) r;
        return (int) (temp + 0.5);
    }

    public static explicit operator float (RationalNumber r)
    {
        return ((float) r.numerator) / r.denominator;
    }

    public static implicit operator RationalNumber (int n)
    {
        return new RationalNumber (n, 1);
    }

    public override String ToString ()
    {
        return String.Format ("{0}/{1}", numerator, denominator);
    }
}

class Demo
{
    public static void Main ()
    {  
        Console.WriteLine ("testing unary arithmetic operators");
        RationalNumber rn = new RationalNumber (1,7);
        Console.WriteLine ("rn = {0}", rn);
        Console.WriteLine ("+rn = {0}", +rn);
        Console.WriteLine ("-rn = {0}", -rn);

        ++ rn;
        Console.WriteLine ("++rn = {0}", rn);
        rn --;
        Console.WriteLine ("rn-- = {0}", rn);

        Console.WriteLine ("++rn = {0}", ++rn);
        Console.WriteLine ("rn-- = {0}", rn--);
        Console.WriteLine ("rn = {0}", rn);
        Console.WriteLine ();
        
        Console.WriteLine ("testing binary arithmetic operators");
        RationalNumber rn1 = new RationalNumber (1,7);
        RationalNumber rn2 = new RationalNumber (2,7);
        Console.WriteLine ("rn1 = {0}", rn1);
        Console.WriteLine ("rn2 = {0}", rn2);
        Console.WriteLine ("rn1 + rn2 = {0}", rn1 + rn2);
        Console.WriteLine ("rn1 - rn2 = {0}", rn1 - rn2);
        Console.WriteLine ("rn1 * rn2 = {0}", rn1 * rn2);
        Console.WriteLine ("rn1 / rn2 = {0}", rn1 / rn2);
        Console.WriteLine ("rn1 + 1 = {0}", rn1 + 1);
        Console.WriteLine ("rn1 - 1 = {0}", rn1 - 1);
        Console.WriteLine ("rn1 * 1 = {0}", rn1 * 1);
        Console.WriteLine ("rn1 / 1 = {0}", rn1 / 1);
        Console.WriteLine ("1 + rn1 = {0}", 1 + rn1);
        Console.WriteLine ("1 - rn1 = {0}", 1 - rn1);
        Console.WriteLine ("1 * rn1 = {0}", 1 * rn1);
        Console.WriteLine ("1 / rn1 = {0}", 1 / rn1);
        Console.WriteLine ();

        Console.WriteLine ("testing comparison operators");
        rn1 = new RationalNumber (3,7);
        rn2 = new RationalNumber (5,7);
        Console.WriteLine ("rn1 = {0}", rn1);
        Console.WriteLine ("rn2 = {0}", rn2);
        Console.WriteLine ("rn1 == rn2: {0}", rn1 == rn2);
        Console.WriteLine ("rn1 != rn2: {0}", rn1 != rn2);
        Console.WriteLine ("rn1 < rn2: {0}", rn1 < rn2);
        Console.WriteLine ("rn1 > rn2: {0}", rn1 > rn2);
        Console.WriteLine ("rn1 <= rn2: {0}", rn1 <= rn2);
        Console.WriteLine ("rn1 >= rn2: {0}", rn1 >= rn2);
        Console.WriteLine ();

        Console.WriteLine ("testing conversion operators");
        rn = new RationalNumber (1, 3);
        Console.WriteLine ("rn = {0}", rn);
        Console.WriteLine ("(int) rn = {0}", (int) rn);
        Console.WriteLine ("(float) rn = {0}", (float) rn);
        rn = 7;
        Console.WriteLine ("rn = 7: {0}", rn);
        Console.WriteLine ();
    }
}

#if TEST
class RationalNumber
{
    private int numerator;    // dividend
    private int denominator;  // divisor

    // c'tors
    public RationalNumber (int numerator, int denominator)
    {
        if (denominator == 0)
        {
            throw new ArgumentOutOfRangeException ("Divisor cannot be zero");
        }
        else
        {
            this.numerator = numerator;
            this.denominator = denominator;
        }
    }

    public RationalNumber (int numerator) : this (numerator, 1)
    {
    }

    public RationalNumber () : this (1)
    {
    }

    public RationalNumber (RationalNumber r) : this (r.numerator, r.denominator)
    {
    }

    // unary operators
    public static RationalNumber operator++ (RationalNumber r)
    {
        int numerator = r.numerator + r.denominator;
        int denominator = r.denominator;
        RationalNumber r1 = new RationalNumber (numerator, denominator);
        return r1;
    }

    public static RationalNumber operator-- (RationalNumber r)
    {
        int numerator = r.numerator - r.denominator;
        int denominator = r.denominator;
        RationalNumber r1 = new RationalNumber (numerator, denominator);
        return r1;
    }

    // binary arithmetic operators
    public static RationalNumber operator+ (RationalNumber r1, RationalNumber r2)
    {
        int numerator =
            r1.numerator * r2.denominator + r2.numerator * r1.denominator;
        int denominator = r1.denominator * r2.denominator;
        RationalNumber r = new RationalNumber (numerator, denominator);
        return r;
    }

    public static RationalNumber operator- (RationalNumber r1, RationalNumber r2)
    {
        int numerator =
            r1.numerator * r2.denominator - r2.numerator * r1.denominator;
        int denominator = r1.denominator * r2.denominator;
        RationalNumber r = new RationalNumber (numerator, denominator);
        return r;
    }

    // comparison operators
    public static bool operator== (RationalNumber r1, RationalNumber r2)
    {
        return (r1.numerator * r2.denominator == r1.denominator * r2.numerator) ?
            true : false;
    }

    public static bool operator!= (RationalNumber r1, RationalNumber r2)
    {
        return ! (r1 == r2);
    }
    
    public static bool operator< (RationalNumber r1, RationalNumber r2)
    {
        return (r1.numerator * r2.denominator ) < (r2.numerator * r1.denominator);
    }

    public static bool operator> (RationalNumber r1, RationalNumber r2)
    {
        return !((r1 < r2) || (r1 == r2));
    }

    public static bool operator<= (RationalNumber r1, RationalNumber r2)
    {
        return !(r1 > r2);
    }

    public static bool operator>= (RationalNumber r1, RationalNumber r2)
    {
        return !(r1 < r2);
    }

    // conversion operators
    public static implicit operator double (RationalNumber r)
    {
        return ((double) r.numerator) / r.denominator;
    }

    public static explicit operator int (RationalNumber r)
    {
        double temp = r;
        return (int) (temp + 0.5);
    }

    public static implicit operator String (RationalNumber r)
    {
        return r.ToString ();
    }

    public override String ToString ()
    {
        return String.Format ("{0}/{1}", numerator, denominator);
    }
    
    public override bool Equals (Object r)
    {
        if (! (r is RationalNumber))
            return false;

        return this == (RationalNumber) r;
    }

    public override int GetHashCode ()
    {
        return base.GetHashCode ();
    }    
}

class RationalNumberDemo
{
    public static void Main ()
    {
        // testing exception handling
        try
        {
            RationalNumber rn = new RationalNumber (3,0);
        }
        catch (ArgumentOutOfRangeException)
        {
            Console.WriteLine ("creation of a rational number failed !!!");
        }

        // create a couple of rational numbers
        RationalNumber rn1 = new RationalNumber (1,7);
        RationalNumber rn2 = new RationalNumber (3);
        RationalNumber rn3 = new RationalNumber (rn1);
        RationalNumber rn4 = new RationalNumber ();

        // print them out
        Console.WriteLine ("rn1: {0}", rn1);
        Console.WriteLine ("rn2: {0}", rn2);
        Console.WriteLine ("rn3: {0}", rn3);
        Console.WriteLine ("rn4: {0}", rn4);

        // testing unary operators
        Console.WriteLine ("rn1: {0}", rn1);
        rn1 ++;
        Console.WriteLine ("rn1: {0}", rn1);
        rn1 --;
        Console.WriteLine ("rn1: {0}", rn1);

        // testing arithmetic operators
        RationalNumber rn10 = rn1 + rn2;
        Console.WriteLine ("rn1 + rn2: {0}", rn10);

        RationalNumber rn11 = rn4 - rn3;
        Console.WriteLine ("rn4 - rn3: {0}", rn11);

        // testing comparison operators
        if (rn1 > rn2) 
        {
            Console.WriteLine("{0} > {1}", rn1, rn2);
        } 
        else 
        {
            Console.WriteLine("{0} is not > {1}", rn1, rn2);
        }

        // is rn1 <= rn2?
        if (rn1 <= rn2) 
        {
            Console.WriteLine("{0} <= {1}", rn1, rn2);
        } 
        else 
        {
            Console.WriteLine("{0} is not <= {1}", rn1, rn2);
        }

        // is rn1 != rn2?
        if (rn1 != rn2) 
        {
            Console.WriteLine("{0} != {1}", rn1, rn2);
        } 
        else 
        {
            Console.WriteLine("{0} is not != {1}", rn1, rn2);
        }

        // testing implicit conversion operator 'double' (value type)
        double d = rn1;  // no cast required, operator is defined 'implicit'
        Console.WriteLine("d = {0}", d);    
    
        // testing explicit conversion operator 'int' (value type)
        int n = (int) rn1;  // cast required, operator is defined 'explicit'
        Console.WriteLine("n = {0}", n);

        // testing implicit conversion operator 'String' (reference type)
        String s = rn1;  // no cast required, operator is defined 'implicit'
        Console.WriteLine("s = {0}", s);
   
    }
}
#endif