using System;

[AttributeUsage (
     AttributeTargets.Class |
     AttributeTargets.Struct |
     AttributeTargets.Enum |
     AttributeTargets.Interface,
     AllowMultiple = false)
]
class DeveloperInfoAttribute : Attribute
{
    private String name;
    private String date;
    private String comment;

    // c'tors
    public DeveloperInfoAttribute (String name)
    {
        this.name = name;
        this.date = "";
        this.comment = "";
    }

    public DeveloperInfoAttribute (String name, String date)
    {
        this.name = name;
        this.date = date;
        this.comment = "";
    }

    // properties for named parameters
    public String Comment
    {
        set
        {
            this.comment = value;
        }

        get
        {
            return this.comment;
        }
    }

    // read-only properties for positional parameters
    public String Name
    {
        get
        {
            return this.name;
        }
    }

    public String Date
    {
        get
        {
            return this.date;
        }
    }
}

[DeveloperInfo ("Peter Loos")]
class A
{
}

[DeveloperInfo ("Peter Loos", "10-11-05")]
class B
{
}

[DeveloperInfo ("Peter Loos", Comment = "Initial revision")]
class C
{
}

class AttributeTest
{
    public static void PrintDeveloperInfoAttribute (Type target)
    {
        // obtain custom attributes
        Type typ = typeof (DeveloperInfoAttribute);
        Object[] attrs = target.GetCustomAttributes (typ, false);

        for (int i = 0; i < attrs.Length; i ++)
        {
            DeveloperInfoAttribute dia =
                (DeveloperInfoAttribute) attrs[i];

            String s = String.Format ("Name: {0}", dia.Name);
            if (dia.Date != "")
                s = s + String.Format (", Date: {0}", dia.Date);
            if (dia.Comment != "")
                s = s + String.Format (", Comment: {0}", dia.Comment);
            Console.WriteLine (s);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        AttributeTest.PrintDeveloperInfoAttribute (typeof (A));
        AttributeTest.PrintDeveloperInfoAttribute (typeof (B));
        AttributeTest.PrintDeveloperInfoAttribute (typeof (C));
    }
}

