using System;

class Time
{
    private int second;
    private int minute;
    private int hour;

    public Time (int second, int minute, int hour)
    {
        this.second = second;
        this.minute = minute;
        this.hour = hour;
    }

    public override String ToString ()
    {
        return String.Format ("{0:00}:{1:00}:{2:00}",
            this.hour, this.minute, this.second);
    }

    public override bool Equals (Object o)
    {
        if (o == null)
            return false;

          // comparing run-time types
//        if (! (o is Time))
//            return false;

        Type typ = this.GetType ();
        if (! typ.Equals (o.GetType ()))
            return false;

        Time t = (Time) o;
        if (this.second != t.second)
            return  false;
        if (this.minute != t.minute)
            return  false;
        if (this.hour != t.hour)
            return  false;

        return true;
    }

    public override int GetHashCode ()
    {
        return (int) this.second |
            (int) (this.minute << 8) |
            (int) (this.hour << 16);
    }
    
    public  static bool operator == (Time t1, Time t2)
    {
        return t1.Equals (t2);
    }

    public  static bool operator != (Time t1, Time t2)
    {
        return ! (t1 == t2);
    }
}

class LocalTime : Time
{
    public LocalTime (int second, int minute, int hour) :
        base (second, minute, hour)
    {
    }
}

class Demo
{
    public static void Demo01 ()
    {
        Time t1 = new Time (0, 0, 12);
        Time t2 = new Time (0, 0, 12);

        Console.WriteLine (t1.Equals (t2));
    }

    public static void Demo02 ()
    {
        Time t1 = new Time (0, 0, 12);
        Time t2 = new Time (0, 0, 12);

        // comparing references to separate objects
        bool b = Object.ReferenceEquals (t1, t2);
        Console.WriteLine (b);

        t2 = t1;

        // comparing references to the same object
        b = Object.ReferenceEquals (t1, t2);
        Console.WriteLine (b);
    }

    public static void Demo03 ()
    {
        Time t1 = new Time (0, 30, 9);
        Console.WriteLine (t1);

    }

    public static void Demo04 ()
    {
        Object o = new Time (0, 0, 0);
        Time t = new Time (0, 0, 0);

        Type typ1 = o.GetType ();
        Type typ2 = t.GetType ();

        Console.WriteLine (typ1.ToString ());
        Console.WriteLine (typ2.ToString ());
    }

    public static void Demo05 ()
    {
        Time t1 = new Time (59, 59, 23);
        Time t2 = new Time (59, 59, 23);
        Console.WriteLine (t1.GetHashCode ());
        Console.WriteLine (t2.GetHashCode ());
    }

    public static void Demo06 ()
    {
        Time t1 = new Time (59, 59, 23);
        Time t2 = new LocalTime (59, 59, 23);

        Console.WriteLine (t1.Equals (t2));
    }

    public static void Demo07 ()
    {
        Time t1 = new Time (0, 0, 0);
        LocalTime t2 = new LocalTime (0, 0, 0);
        Type typ1 = t1.GetType ();
        Type typ2 = t2.GetType ();

        Console.WriteLine (typ1.Equals (typ2));
        Console.WriteLine (t2 is Time);
    }

    public static void Demo08 ()
    {
        Time t = new Time (0, 0, 0);
        Type typ = t.GetType ();

        Type typ1 = typeof (Time);

        Console.WriteLine (typ.Equals (typ1));
    }

    public static void Main ()
    {
        Demo01 ();
    }
}