#define VARIANT_01         // IComparable
// #define VARIANT_02      // IComparer

using System;
using System.Collections;

#if VARIANT_01
class Time : IComparable
{
    public int hour;
    public int minute;
    public int second;

    public Time (int hour, int minute, int second)
    {
        this.hour = hour;
        this.minute = minute;
        this.second = second;
    }

    // implementation of interface 'IComparable'
    public int CompareTo (Object obj)
    {
        if (obj is Time) 
        {
            Time t = (Time) obj;

            if (this.hour < t.hour)
                return -1;
            else if (this.hour == t.hour)
            {
                if (this.minute < t.minute)
                    return -1;
                else if (this.minute == t.minute)
                {
                    return (this.second < t.second) ? -1 :
                           (this.second > t.second) ? 1  : 0;
                }
            }
            else
                return 1;
        }

        throw new ArgumentException ("object is not a Time");
    }

    public override String ToString ()
    {
        return String.Format ("[{0:00}:{1:00}:{2:00}]",
            this.hour, this.minute, this.second);
    }
}

class Demo
{
    public static void Main ()
    {
        // 1. example
        Time t1 = new Time (12, 0, 10);
        Time t2 = new Time (12, 0, 11);

        Console.WriteLine (t1.CompareTo (t2));
        Console.WriteLine (t2.CompareTo (t1));
        Console.WriteLine (t1.CompareTo (t1));

        // 2. example
        Time[] times = new Time[5];
        times[0] = new Time (11, 59, 59);
        times[1] = new Time (12,  0,  1);
        times[2] = new Time (12,  0,  0);
        times[3] = new Time (23, 59, 59);
        times[4] = new Time (0,   0,  0);

        // print unsorted array
        for (int i = 0; i < times.Length; i ++)
            Console.WriteLine (times[i].ToString ());
        Console.WriteLine ();

        // sort array (using IComparable interface)
        Array.Sort (times);

        // print sorted array
        for (int i = 0; i < times.Length; i ++)
            Console.WriteLine (times[i].ToString ());
        Console.WriteLine ();
    }
}
#endif


#if VARIANT_02
class Time
{
    public int hour;
    public int minute;
    public int second;

    public Time (int hour, int minute, int second)
    {
        this.hour = hour;
        this.minute = minute;
        this.second = second;
    }

    public override String ToString ()
    {
        return String.Format ("[{0:00}:{1:00}:{2:00}]",
            this.hour, this.minute, this.second);
    }
}

class SortBySecond : IComparer
{
    public int Compare (Object o1, Object o2)
    {
        Time t1 = (Time) o1;
        Time t2 = (Time) o2;

        return (t1.second < t2.second) ? -1 :
            (t1.second > t2.second) ? 1 : 0;
    }
}

class Demo
{
    public static void Main ()
    {
        // 3. example
        Time[] times = new Time[5];
        times[0] = new Time (11, 59, 59);
        times[1] = new Time (12,  0,  1);
        times[2] = new Time (12,  0,  0);
        times[3] = new Time (23, 59, 59);
        times[4] = new Time (0,   0,  0);

        // helper class, implementing IComparer interface
        IComparer icomparer = new SortBySecond ();

        // sort array (using IComparer interface)
        Array.Sort (times, icomparer);

        // print sorted array
        for (int i = 0; i < times.Length; i ++)
            Console.WriteLine (times[i].ToString ());
        Console.WriteLine ();
    }
}
#endif
