using System;

interface IComputation
{
    double ComputeArea ();
    double ComputeCircumference ();
}

class Rectangle : IComputation
{
    private int x;
    private int y;
    private int width;
    private int height;

    public Rectangle (int x, int y, int width, int height)
    {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }

    public double ComputeArea ()
    {
        return this.width * this.height;
    }

    public double ComputeCircumference ()
    {
        return 2 * (this.width + this.height);
    }    

    public void Print ()
    {
        Console.WriteLine ("[{0},{1},{2},{3}]",
            this.x, this.y, this.width, this.height);
    }    
}

class Circle : IComputation
{
    private int x;
    private int y;
    private double radius;

    public Circle (int x, int y, int radius)
    {
        this.x = x;
        this.y = y;
        this.radius = radius;
    }

    public double ComputeArea ()
    {
        return Math.PI * this.radius * this.radius;
    }

    public double ComputeCircumference ()
    {
        return 2 * Math.PI * this.radius;
    }    
}

class Demo
{
    public static void Main ()
    {
        // create some concrete objects
        Circle c1 = new Circle (50, 50, 10);
        Rectangle r1 = new Rectangle (1, 1, 20, 40);
        Rectangle r2 = new Rectangle (10, 10, 20, 40);

        // create an array to hold 'computational' objects
        IComputation[] shapes = new IComputation[3];

        // remember concrete objects to fulfil their 'computational' contract
        shapes[0] = c1;
        shapes[1] = r1;
        shapes[2] = r2;

        // compute total area of all these shapes
        double totalArea = 0.0;
        for (int i = 0; i < shapes.Length; i ++)
            totalArea += shapes[i].ComputeArea ();
        Console.WriteLine ("Total area: {0}", totalArea);

        Console.WriteLine ("Total area: {0}", ComputeTotalArea (shapes));

        // compute total circumference of all these shapes
        double totalCircumference = 0.0;
        for (int i = 0; i < shapes.Length; i ++)
            totalCircumference += shapes[i].ComputeCircumference ();
        Console.WriteLine ("Total circumference: {0}", totalCircumference);
    }

    public static double ComputeTotalArea (IComputation[] shapes)
    {
        double totalArea = 0.0;
        for (int i = 0; i < shapes.Length; i ++)
            totalArea += shapes[i].ComputeArea ();
        return totalArea;
    }
}
