#define VERSION_01  // generic set for value types
// #define VERSION_02  // generic set for reference types
// #define VERSION_03  // generic set for value & reference types

using System;
using System.Text;

#if VERSION_01
class Set<T> where T : IComparable
{
    private T[] elements;
    private int count;

    // c'tors
    public Set () : this (16)
    {
    }

    public Set (int capacity)
    {
        // allocate array
        this.elements = new T[capacity];
        this.count = 0;

        // initialize array
        for (int i = 0; i < this.elements.Length; i ++)
            this.elements[i] = default (T);
    }

    // properties
    public int Count
    {
        get
        {
            return this.count;
        }
    }

    public bool Empty
    {
        get
        {
            return this.count == 0;
        }
    }
    
    // public methods
    public bool Insert (T val)
    {
        // element already exists
        if (this.Contains (val))
            return false;

        // search empty slot
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (this.elements[i].CompareTo (default (T)) == 0)
            {
                this.elements[i] = val;
                this.count ++;
                return true;
            }
        }

        return false;
    }

    public bool Contains (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
            if (this.elements[i].CompareTo (val) == 0)
                return true;

        return false;
    }

    public bool Remove (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
            if (this.elements[i].CompareTo (val) == 0)
            {
                this.elements[i] = default (T);
                this.count --;
                return true;
            }

        return false;
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (128);
        sb.Append ("[");
        int commas = 0;
        for (int i = 0; i < this.elements.Length; i++)
        {
            if (this.elements[i].CompareTo (default (T)) != 0)
            {
                sb.Append (this.elements[i]);
                commas++;
                if (commas < this.count)
                    sb.Append (", ");
            }
        }
        sb.Append ("]");

        return sb.ToString ();
    }
}
#endif

#if VERSION_02
class Set<T> where T : IComparable
{
    private T[] elements;
    private int count;

    // c'tors
    public Set () : this (16)
    {
    }

    public Set (int capacity)
    {
        // allocate array
        this.elements = new T[capacity];
        this.count = 0;

        // initialize array
        for (int i = 0; i < this.elements.Length; i ++)
            this.elements[i] = default (T);
    }

    // properties
    public int Count
    {
        get
        {
            return this.count;
        }
    }

    public bool Empty
    {
        get
        {
            return this.count == 0;
        }
    }

    // public methods
    public bool Insert (T val)
    {
        // element already exists
        if (this.Contains (val))
            return false;

        // search empty slot
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (this.elements[i] is System.ValueType)
            {
                if (this.elements[i].CompareTo (default (T)) != 0)
                    continue;
            }
            else
            {
                if (this.elements[i] != null)
                    continue;
            }
            
            this.elements[i] = val;
            this.count ++;
            return true;
        }

        return false;
    }

    public bool Contains (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (! (this.elements[i] is System.ValueType))
            {
                if (this.elements[i] == null)
                    continue;
            }

            if (this.elements[i].CompareTo (val) == 0)
                return true;
        }

        return false;
    }

    public bool Remove (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (! (this.elements[i] is System.ValueType))
            {
                if (this.elements[i] == null)
                    continue;
            }

            if (this.elements[i].CompareTo (val) == 0)
            {
                this.elements[i] = default (T);
                this.count --;
                return true;
            }
        }

        return false;
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (128);
        sb.Append ("[");
        int commas = 0;
        for (int i = 0; i < this.elements.Length; i++)
        {
            if (this.elements[i] is System.ValueType)
            {
                if (this.elements[i].CompareTo (default (T)) == 0)
                    continue;
            }
            else
            {
                if (this.elements[i] == null)
                    continue;
            } 
            
            sb.Append (this.elements[i]);
            commas++;
            if (commas < this.count)
                sb.Append (", ");
        }
        sb.Append ("]");

        return sb.ToString ();
    }
}
#endif

#if VERSION_03
class Set<T> where T : IComparable
{
    private Element[] elements;
    private int count;

    // c'tors
    public Set () : this (16)
    {
    }

    public Set (int capacity)
    {
        // allocate array
        this.elements = new Element[capacity];
        this.count = 0;

        // initialize array
        for (int i = 0; i < this.elements.Length; i ++)
            this.elements[i] = null;
    }

    // internal helper class
    private class Element
    {
        private T elem;

        // c'tor
        public Element (T elem)
        {
            this.elem = elem;
        }

        // property
        public T Elem
        {
            get
            {
                return this.elem;
            }
        }
    }

    // properties
    public int Count
    {
        get
        {
            return this.count;
        }
    }

    public bool Empty
    {
        get
        {
            return this.count == 0;
        }
    }

    // public methods
    public bool Insert (T val)
    {
        // element already exists
        if (this.Contains (val))
            return false;

        // search empty slot
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (this.elements[i] == null)
            {
                this.elements[i] = new Element (val);
                this.count ++;
                return true;
            }
        }

        return false;
    }

    public bool Contains (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (this.elements[i] == null)
                continue;

            T elem = this.elements[i].Elem;
            if (elem.CompareTo (val) == 0)
                return true;
        }

        return false;
    }

    public bool Remove (T val)
    {
        for (int i = 0; i < this.elements.Length; i ++)
        {
            if (this.elements[i] == null)
                continue;

            T elem = this.elements[i].Elem;
            if (elem.CompareTo (val) == 0)
            {
                this.elements[i] = null;
                this.count --;
                return true;
            }
        }

        return false;
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (128);
        sb.Append ("[");
        int commas = 0;
        for (int i = 0; i < this.elements.Length; i++)
        {
            if (this.elements[i] == null)
                continue;

            T elem = this.elements[i].Elem;
            sb.Append (elem);
            commas++;
            if (commas < this.count)
                sb.Append (", ");
        }
        sb.Append ("]");

        return sb.ToString ();
    }
}
#endif

class Demo
{
    public static void Main ()
    {
        Set<int> s1 = new Set<int> ();
        s1.Insert (11);
        s1.Insert (12);
        Console.WriteLine (s1);
        s1.Remove (12);
        Console.WriteLine (s1);

        Set<double> s2 = new Set<double> ();
        s2.Insert (1.1);
        s2.Insert (2.2);
        Console.WriteLine (s2);
        s2.Remove (1.1);
        Console.WriteLine (s2);

#if VERSION_02 || VERSION_03
        Set<String> s3 = new Set<String> ();
        s3.Insert ("One");
        s3.Insert ("Two");
        s3.Insert ("Three");
        Console.WriteLine (s3);
        s3.Remove ("Two");
        Console.WriteLine (s3);
#endif
    }
}

