using System;

delegate int Comparison (Object l, Object r);

class Employee
{
    private String name;
    private int age;
    private decimal salary;

    public Employee (String name, int age, decimal salary)
    {
        this.name = name;
        this.age = age;
        this.salary = salary;
    }

    public override String ToString ()
    {
        String text = String.Format ("Name: {0}, Age: {1}, Salary: {2}",
            this.name, this.age, this.salary);
        return text;
    }

    public static int CompareByName (Object l, Object r)
    {
        String ln = ((Employee) l).name;
        String rn = ((Employee) r).name;
        return String.Compare (ln, rn);
    }

    public static int CompareByAge (Object l, Object r)
    {
        int la = ((Employee) l).age;
        int ra = ((Employee) r).age;
        return (la < ra) ? -1 : (la > ra) ?  1 : 0;
    }

    public static int CompareBySalary (Object l, Object r)
    {
        decimal ls = ((Employee) l).salary;
        decimal rs = ((Employee) r).salary;
        return (ls < rs) ? -1 : (ls > rs) ?  1 : 0;
    }
}

class BubbleSorter
{
    // classical bubble sort algorithm
    public static void Sort (int[] array)
    {
        for (int i = array.Length - 1; i >= 0; i --) 
        {
            bool swapped = false;

            for (int j = 0; j < i; j ++) 
            {
                // swap two elements, if necessary
                if (array[j+1] < array[j]) 
                {
                    int tmp = array[j];
                    array[j] = array[j+1];
                    array[j+1] = tmp;

                    // continue sorting
                    swapped = true;
                }
            }

            if (! swapped)
            {
                // array is sorted
                break;
            }
        }
    }

    // generic bubble sort algorithm
    public static void Sort (Object[] array, Comparison Compare)
    {
        for (int i = array.Length - 1; i >= 0; i --) 
        {
            bool swapped = false;

            for (int j = 0; j < i; j ++) 
            {
                // swap two elements, if necessary
                if (Compare (array[j+1], array[j]) < 0)
                {
                    // swap elements
                    Object tmp = array[j];
                    array[j] = array[j+1];
                    array[j+1] = tmp;

                    // continue sorting
                    swapped = true;
                }
            }

            if (! swapped)
            {
                // array is sorted
                break;
            }
        }
    }
}

class Int32Comparison
{
    public static int Compare (Object l, Object r)
    {
        int left = (int) l;
        int right = (int) r;
        return (left < right) ? -1 : (left > right) ?  1 : 0;
    }
}

class Examples
{
    public static void Example01 ()
    {
        Employee[] employees =
        {
            new Employee ("Franz Huber", 41, 1500.00M),
            new Employee ("Hubert Mueller", 27, 1000.00M),
            new Employee ("Heinz Meier", 35, 800M),
            new Employee ("Susanne Schwarz", 57, 5000M)
        };

        Console.WriteLine ("Employees:");
        for (int i = 0; i < employees.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, employees[i].ToString ());
        Console.WriteLine ();

        // testing generic bubble sort algorithm (sorting by salary)
        Console.WriteLine ("Sorting by salary:");
        Comparison d = new Comparison (Employee.CompareBySalary);
        BubbleSorter.Sort (employees, d);
        for (int i = 0; i < employees.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, employees[i].ToString ());
        Console.WriteLine ();

        // testing generic bubble sort algorithm (sorting by name)
        Console.WriteLine ("Sorting by name:");
        d = new Comparison (Employee.CompareByName);
        BubbleSorter.Sort (employees, d);
        for (int i = 0; i < employees.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, employees[i].ToString ());
        Console.WriteLine ();

        // testing generic bubble sort algorithm (sorting by age)
        Console.WriteLine ("Sorting by age:");
        d = new Comparison (Employee.CompareByAge);
        BubbleSorter.Sort (employees, d);
        for (int i = 0; i < employees.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, employees[i].ToString ());
        Console.WriteLine ();
    }

    public static void Example02 ()
    {
        // testing generic bubble sort algorithm using boxed 'int' objects
        Object[] someInts = {5, 4, 3, 2, 1};

        for (int i = 0; i < someInts.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, someInts[i]);
        Console.WriteLine ();

        Comparison compareInt32s = new Comparison (Int32Comparison.Compare);
        BubbleSorter.Sort (someInts, compareInt32s);

        for (int i = 0; i < someInts.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, someInts[i]);
        Console.WriteLine ();  
    }

    public static void Example03 ()
    {
        // testing classical bubble sort algorithm
        int[] someInts = {5, 4, 3, 2, 1};

        for (int i = 0; i < someInts.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, someInts[i]);
        Console.WriteLine ();

        BubbleSorter.Sort (someInts);

        for (int i = 0; i < someInts.Length; i ++)
            Console.WriteLine ("{0}: {1}", i, someInts[i]);
        Console.WriteLine ();
    }
}
