using System;

public delegate int MyDelegate (String s);

public delegate void PrintString ();

class StringBox
{
    private String s;

    public StringBox (String s)
    {
        this.s = s;
    }

    public void Print ()
    {
        Console.WriteLine ("Sring '{0}' is {1} bytes long",
            this.s, this.s.Length);
    }
}

class SimpleDelegateDemo
{
    public static void Example01 ()
    {
        String s = "12345";
        MyDelegate d = new MyDelegate (s.IndexOf);
        int pos = d ("3");
        
        Console.WriteLine (pos);
    }

    public static void Example02 ()
    {
        // create a couple of 'StringBox' objects
        StringBox sc1 = new StringBox ("A");
        StringBox sc2 = new StringBox ("simple");
        StringBox sc3 = new StringBox ("String");

        // create a couple of delegate objects
        PrintString d1 = new PrintString (sc1.Print);
        PrintString d2 = new PrintString (sc2.Print);
        PrintString d3 = new PrintString (sc3.Print);

        // create a multicast delegate using 'MulticastDelegate.Combine'
        PrintString m = (PrintString) MulticastDelegate.Combine (d1, d2);
        m = (PrintString) MulticastDelegate.Combine (m, d3);

        // iterate through multi's invocation list explicitly
        Delegate[] list = m.GetInvocationList ();
        for (int i = 0; i < list.Length; i ++)
        {
            ((PrintString) list [i]) ();
        }
    }
    
    public static void Example03 ()
    {
        // create a couple of 'StringBox' objects
        StringBox sc1 = new StringBox ("A");
        StringBox sc2 = new StringBox ("simple");
        StringBox sc3 = new StringBox ("String");

        // create a couple of delegate objects
        PrintString d1 = new PrintString (sc1.Print);
        PrintString d2 = new PrintString (sc2.Print);
        PrintString d3 = new PrintString (sc3.Print);

        // create a multicast delegate
        PrintString m = d1 + d2;
        m += d3;

        // invoke delegate
        m ();
    }
    
    public delegate String TwoStringParameters (String s1, String s2);
    public delegate String TwoIntParameters (int n, int m);

    public static void IllegalArgumentDemo ()
    {
        // create a 'String' object
        String s = "Hello World";

        // create a 'TwoStringParameters' delegate object
        TwoStringParameters d1 = new TwoStringParameters (s.Replace);

        // create a 'TwoIntParameters' delegate object
        TwoIntParameters d2 = new TwoIntParameters (s.Substring);

        // create a multicast delegate using 'MulticastDelegate.Combine'
        // (Ooops: second parameter has wrong type !)
        TwoStringParameters m =
            (TwoStringParameters) MulticastDelegate.Combine (d1, d2);
    }
    
    public static void Main ()
    {
        Example01 ();
    }
}
