using System;
using System.Collections;

// class to encapsulate data for a 'PointAdded' event
class PointAddedEventArgs : EventArgs
{
    private int x;
    private int y;

    public PointAddedEventArgs (int x, int y)
    {
        this.x = x;
        this.y = y;
    }

    public int X
    {
        get
        {
            return this.x;
        }
    }

    public int Y
    {
        get
        {
            return this.y;
        }
    }
}

class Point
{
    public int x;
    public int y;

    public Point (int x, int y)
    {
        this.x = x;
        this.y = y;
    }
}

// delegate declaration for 'PointAdded' event
delegate void PointAddedEventHandler (Object sender, PointAddedEventArgs e);

class Polygon
{
    public event PointAddedEventHandler PointAdded;

    private ArrayList list;

    public Polygon ()
    {
        this.PointAdded = null;
        this.list = new ArrayList ();
    }

    public void add_PointAdded0 (PointAddedEventHandler handler)
    {
        this.PointAdded = (PointAddedEventHandler)
            MulticastDelegate.Combine (this.PointAdded, handler);
    }

    public void remove_PointAdded0 (PointAddedEventHandler handler)
    {
        this.PointAdded = (PointAddedEventHandler)
            MulticastDelegate.Remove (this.PointAdded, handler);
    }

    public bool AddPoint (int x, int y)
    {
        for (int i = 0; i < list.Count; i ++)
        {
            Point p = (Point) list [i];
            if (p.x == x && p.y == y)
                return false;
        }

        Console.WriteLine
            ("[Publisher]  adding point {0},{1} to polygon.", x, y);

        Point q = new Point (x, y);
        list.Add (q);

        // notify subscribed clients
        if (PointAdded != null) 
        {
            PointAddedEventArgs ev = new PointAddedEventArgs (x, y);
            PointAdded (this, ev);
        }

        return true;
    }
}

class PolygonClient
{
    public void PointWasAdded (Object sender, PointAddedEventArgs e)
    {
        int x = e.X;
        int y = e.Y;
        Console.WriteLine (
            "[Subscriber] point {0},{1} added to polygon.", x, y);
    }
}


class Test
{
    public static void Main ()
    {
        Example ();
    }
        
    public static void Example ()
    {
        // create a Polygon (event source)
        Polygon p = new Polygon ();

        // create a client (event sink)
        PolygonClient client = new PolygonClient ();

        // create a 'PointAddedEventHandler' delegate
        PointAddedEventHandler handler =
            new PointAddedEventHandler (client.PointWasAdded);

        // subscribe client to the Polygon's PointAdded-event
//        p.PointAdded += handler;
        p.add_PointAdded0 (handler);

        // add a couple of points to the polygon
        p.AddPoint (5, 7);
        p.AddPoint (1, 9);
        p.AddPoint (4, 6);

        // unsubscribe client
//        p.PointAdded -= handler;
        p.remove_PointAdded0 (handler);

        p.AddPoint (2, 8);
        p.AddPoint (3, 5);
    }
}
