#define VARIANTE_01
// #define VARIANTE_02   // using standard event conventions
// #define VARIANTE_03   // using an anonymous method
// #define VARIANTE_04   // using an anonymous method

#if VARIANTE_01 || VARIANTE_03 || VARIANTE_04

using System;
using System.Collections;

// delegate declaration for 'PointAdded' event
delegate void PointAddedEventHandler (int x, int y);

class Point
{
    public int x;
    public int y;

    public Point (int x, int y)
    {
        this.x = x;
        this.y = y;
    }
}

class Polygon
{
    public event PointAddedEventHandler PointAdded;

    private ArrayList list;

    public Polygon ()
    {
        this.PointAdded = null;
        this.list = new ArrayList ();
    }

    public bool AddPoint (int x, int y)
    {
        for (int i = 0; i < this.list.Count; i ++)
        {
            Point p = (Point) this.list [i];
            if (p.x == x && p.y == y)
                return false;
        }

        Console.WriteLine
            ("[Publisher]  adding point {0},{1} to polygon.", x, y);

        Point q = new Point (x, y);
        this.list.Add (q);

        // notify subscribed clients
        if (this.PointAdded != null)
            this.PointAdded (x, y);

        return true;
    }

    public override String ToString ()
    {
        String s = "[";

        if (this.list.Count == 0)
        {
            s += " <empty> ";
        }
        else
        {
            for (int i = 0; i < this.list.Count; i ++)
            {
                Point p = (Point) this.list [i];
                String s1 = String.Format ("({0},{1})", p.x, p.y);
                s += (i == 0) ? s1 : "," + s1;
            }
        }

        s += "]";
        return s;
    }
}

#endif

#if VARIANTE_01

class PolygonClient
{
    public void PointWasAdded (int x, int y)
    {
        Console.WriteLine
            ("[Subscriber] point {0},{1} added to polygon.", x, y);
    }
}

class Demo
{
    public static void Main ()
    {
        // create a Polygon (event source)
        Polygon p = new Polygon ();

        // create a client (event sink)
        PolygonClient client = new PolygonClient ();

        // create a 'PointAddedEventHandler' delegate
        PointAddedEventHandler handler =
            new PointAddedEventHandler (client.PointWasAdded);

        // subscribe client to the polygons 'PointAdded'-event
        p.PointAdded += handler;

        // add a couple of points to the polygon
        p.AddPoint (5, 7);
        p.AddPoint (1, 9);
        p.AddPoint (4, 6);

        // unsubscribe client
        p.PointAdded -= handler;

        p.AddPoint (2, 8);
        p.AddPoint (3, 5);

        Console.WriteLine (p);
    }
}

#endif

// =======================================================================

#if VARIANTE_02

using System;
using System.Collections;

/*
 *  2. Variante (konform zum .NET-Ereignismodell) 
 */

// class to encapsulate data for a 'PointAdded' event
class PointAddedEventArgs : EventArgs
{
    private int x;
    private int y;

    public PointAddedEventArgs (int x, int y)
    {
        this.x = x;
        this.y = y;
    }

    public int X
    {
        get
        {
            return this.x;
        }
    }

    public int Y
    {
        get
        {
            return this.y;
        }
    }
}

class Point
{
    public int x;
    public int y;

    public Point (int x, int y)
    {
        this.x = x;
        this.y = y;
    }
}

// delegate declaration for 'PointAdded' event
delegate void PointAddedEventHandler (Object o, PointAddedEventArgs e);

// event source
class Polygon
{
    public event PointAddedEventHandler PointAdded;

    private ArrayList list;

    public Polygon ()
    {
        this.PointAdded = null;
        this.list = new ArrayList ();
    }

    public bool AddPoint (int x, int y)
    {
        for (int i = 0; i < this.list.Count; i ++)
        {
            Point p = (Point) this.list [i];
            if (p.x == x && p.y == y)
                return false;
        }

        Console.WriteLine
            ("[Publisher]  adding point {0},{1} to polygon.", x, y);

        Point q = new Point (x, y);
        this.list.Add (q);

        // notify subscribed clients
        PointAddedEventArgs e = new PointAddedEventArgs (x, y);
        this.OnPointAdded (e);

        return true;
    }

    protected virtual void OnPointAdded (PointAddedEventArgs e)
    {
        if (this.PointAdded != null) 
            this.PointAdded (this, e);
    }

    public override String ToString ()
    {
        String s = "[";

        if (this.list.Count == 0)
        {
            s += " <empty> ";
        }
        else
        {
            for (int i = 0; i < this.list.Count; i ++)
            {
                Point p = (Point) this.list [i];
                String s1 = String.Format ("({0},{1})", p.x, p.y);
                s += (i == 0) ? s1 : "," + s1;
            }
        }

        s += "]";
        return s;
    }
}

// event sink
class PolygonClient
{
    public void PointWasAdded (Object o, PointAddedEventArgs e)
    {
        Console.WriteLine
            ("[Subscriber] point {0},{1} added to polygon.", e.X, e.Y);
    }
}

class Test
{
    public static void Main ()
    {
        // create a Polygon (event source)
        Polygon p = new Polygon ();

        // create a client (event sink)
        PolygonClient client = new PolygonClient ();

        // create a 'PointAddedEventHandler' delegate
        PointAddedEventHandler handler =
            new PointAddedEventHandler (client.PointWasAdded);

        // subscribe client to the polygons 'PointAdded'-event
        p.PointAdded += handler;

        // add a couple of points to the polygon
        p.AddPoint (5, 7);
        p.AddPoint (1, 9);
        p.AddPoint (4, 6);

        // unsubscribe client
        p.PointAdded -= handler;

        p.AddPoint (2, 8);
        p.AddPoint (3, 5);

        Console.WriteLine (p);
    }
}
#endif

#if VARIANTE_03
class Demo
{
    public static void Main ()
    {
        // create a Polygon (event source)
        Polygon p = new Polygon ();

        // subscribe an anonymous method to the polygons 'PointAdded'-event
        p.PointAdded += delegate (int x, int y)
        {
            Console.WriteLine
                ("[Subscriber] point {0},{1} added to polygon.", x, y);
        };

        // add a couple of points to the polygon
        p.AddPoint (5, 7);
        p.AddPoint (1, 9);
        p.AddPoint (4, 6);

        Console.WriteLine (p);
    }
}
#endif

#if VARIANTE_04
class Demo
{
    public static void Main ()
    {
        // create a Polygon (event source)
        Polygon p = new Polygon ();

        // create a client (event sink), using an anonymous method
        PointAddedEventHandler handler = delegate (int x, int y)
        {
            Console.WriteLine
                ("[Subscriber] point {0},{1} added to polygon.", x, y);
        };

        // subscribe anonymous method to the polygons 'PointAdded'-event
        p.PointAdded += handler;

        // add a couple of points to the polygon
        p.AddPoint (5, 7);
        p.AddPoint (1, 9);
        p.AddPoint (4, 6);

        // unsubscribe client
        p.PointAdded -= handler;

        p.AddPoint (2, 8);
        p.AddPoint (3, 5);

        Console.WriteLine (p);
    }
}
#endif
