using System;

enum SwitchPosition {On, Off};

// class to encapsulate switch flipped event
class SwitchFlippedEventArgs : EventArgs
{
    private SwitchPosition position;

    public SwitchFlippedEventArgs (SwitchPosition pos)
    {
        this.position = pos;
    }

    public SwitchPosition Position
    {
        get
        {
            return this.position;
        }
    }
}

// delegate declaration
delegate void
SwitchFlippedEventHandler (Object sender, SwitchFlippedEventArgs e);

// implementation of a switch
class Switch
{
    // event declaration
    public event SwitchFlippedEventHandler SwitchFlipped;

    // current switch state
    private SwitchPosition pos;

    public Switch ()
    {
        this.pos = SwitchPosition.Off;
        this.SwitchFlipped = null;
    }

    // called when switch is flipped on or off
    public void FlipSwitch (SwitchPosition pos)
    {
        Console.WriteLine ("Switch: flipping {0}", pos);

        if (this.pos != pos)
        {
            this.pos = pos;
            
            SwitchFlippedEventArgs e =
                new SwitchFlippedEventArgs (this.pos);

            this.OnSwitchFlipped (e);
        }
    }

    // called when state of switch has changed
    protected virtual void OnSwitchFlipped (SwitchFlippedEventArgs e)
    {
        // invoke the delegate if clients are available
        if (this.SwitchFlipped != null)
            this.SwitchFlipped (this, e);
    }
}

// light object to act as a switch client
class Lamp
{
    private Switch s;

    public Lamp (Switch s)
    {
        this.s = s;
    }

    public void Connect ()
    {
        SwitchFlippedEventHandler handler =
            new SwitchFlippedEventHandler (this.SwitchStateChanged);

        this.s.SwitchFlipped += handler;
    }

    public void Disconnect ()
    {
        SwitchFlippedEventHandler handler =
            new SwitchFlippedEventHandler (this.SwitchStateChanged);

        this.s.SwitchFlipped -= handler;
    }


// BEACHTE:
//    Die beiden Methoden Connect und Disconnect sind korrekt, denn:
//    Es werden zwar zwei UNTERSCHIEDLICHE Objekte an- und abgemeldet,
//    ABER: Entscheidend ist die Objekt- und Methodenreferenz des gekapselten Objekts
//    und nicht die Referenz des delegate-Objekts selbst.


    public void SwitchStateChanged (Object sender, SwitchFlippedEventArgs e)
    {
        SwitchPosition pos = e.Position;
        Console.WriteLine ("Lamp:   switched {0}", pos);
    }
}

// test frame
class Test
{
    public static void Main ()
    {
        // creating a switch
        Switch s = new Switch ();

        // creating a lamp
        Lamp l = new Lamp (s);

        // and connect with switch
        l.Connect ();

        // now flipping switch on
        s.FlipSwitch (SwitchPosition.On);

        // ... and again on
        s.FlipSwitch (SwitchPosition.On);

        // ... and now off
        s.FlipSwitch (SwitchPosition.Off);

        // ... and again off
        s.FlipSwitch (SwitchPosition.Off);

        // ... and finally on
        s.FlipSwitch (SwitchPosition.On);

        // disconnect from switch
        l.Disconnect ();
    }
}
