using System;
using System.Text;

class IntItem
{
    public int elem;
    public IntItem next;

    public IntItem (int elem)
    {
        this.elem = elem;
        this.next = null;
    }
}

class IntList 
{
    private IntItem root;

    // c'tor
    public IntList ()
    {
        this.root = null;
    }

    // properties
    public int Count
    {
        get
        {
            int count = 0;
            for (IntItem node = this.root; node != null; node = node.next)
                count ++;

            return count;
        }
    }

    public bool IsEmpty
    {
        get
        {
            return this.root == null;
        }
    }

    // methods
    public bool Insert (int offset, int value)
    {
        // handle first element seperately
        if (offset == 0)
        {
            if (this.root == null)
            {
                // empty list
                this.root = new IntItem (value);
            }
            else
            {
                IntItem first = new IntItem (value);
                first.next = this.root;
                this.root = first;
            }
            return true;
        }

        // locate position to insert node
        IntItem prev = this.root;
        for (int i = 0; i < offset - 1; i ++)
        {
            if (prev == null)
                return false;

            prev = prev.next;
        }

        // premature end of list reached
        if (prev == null)
            return false;

        // insert node
        IntItem node = new IntItem (value);
        node.next = prev.next;
        prev.next = node;

        return true;
    }
    
    public void Append (int value) 
    {
        // create a new node
        IntItem node = new IntItem (value);

        // append node at end of list
        if (this.root == null)
        {
            this.root = node;
        }
        else
        {
            IntItem last = this.root;
            while (last.next != null)
                last = last.next;

            last.next = node;
        }
    }

    public bool Contains (int value)
    {
        IntItem node = this.root;
        while (node != null)
        {
            if (value == node.elem)
                return true;
            node = node.next;
        }
        return false;
    }

    public bool Remove (int value) 
    {
        IntItem node = this.root;
        IntItem prev = this.root;

        while (node != null)
        {
            if (value == node.elem)
            {
                if (this.root == node)
                {
                    this.root = node.next;
                }
                else
                {
                    prev.next = node.next;
                }
                return true;
            }

            prev = node;
            node = node.next;
        }
        return false;
    }

    // modifying contract with base class 'Object'
    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder ();
        sb.Append ("[");
        IntItem node = this.root;
        while (node != null)
        {
            String format = (node == this.root) ? "{0}" : ", {0}";
            String s = String.Format (format, node.elem);
            sb.Append (s);
            node = node.next;
        }
        sb.Append ("]");
        return sb.ToString ();
    }
}
