#define WINFORMS_01
// #define WINFORMS_02
// #define WINFORMS_03
// #define WINFORMS_04
// #define WINFORMS_05
// #define WINFORMS_06
// #define WINFORMS_07

#if WINFORMS_01
using System.Windows.Forms;

class Demo
{
    public static void Main () 
    {
        Form f = new Form ();
        Application.Run (f);
    }
}
#endif

#if WINFORMS_02

using System.Windows.Forms;
using System.Drawing;

class Demo
{
    public static void Main () 
    {
        Form f = new Form ();

        // set the text associated with this window
        f.Text = "2. Simple WinForms Application";

        // set the size of this window
        f.Height = 100;
        f.Width = 300;

        f.Top = 10;
        f.Left = 10;
        f.StartPosition = FormStartPosition.Manual;

        Application.Run (f);
    }
}
#endif

#if WINFORMS_03
using System;
using System.Drawing;
using System.Windows.Forms;

class Demo
{
    public static void Main ()
    {
        Form f = new Form ();

        // set the text associated with this window
        f.Text = "3. Simple WinForms Application";

        // set the size of this window
        f.Height = 100;
        f.Width = 300;

        // create main menu
        MainMenu mainMenu = new MainMenu();

        // create single "File" menu
        MenuItem fileMenu = new MenuItem ("File");

        // create three menu items for "File" menu
        MenuItem itemNew = new MenuItem ("New");
        MenuItem itemOpen = new MenuItem ("Open");
        MenuItem itemSave = new MenuItem ("Save");

        // add menu items to "File" menu
        fileMenu.MenuItems.Add (itemNew);
        fileMenu.MenuItems.Add (itemOpen);
        fileMenu.MenuItems.Add (itemSave);

        // add "File" menu to menu border
        mainMenu.MenuItems.Add (fileMenu);

        // add menu border to the main form
        f.Menu = mainMenu;

        Application.Run (f);
    }
}
#endif

#if WINFORMS_04
using System;
using System.Drawing;
using System.Windows.Forms;

class Demo
{
    public static void Main ()
    {
        MyForm f = new MyForm ();
        Application.Run (f);
    }
}

class MyForm : Form
{
    // default c'tor
    public MyForm ()
    {
        // set caption bar text and size of this form
        this.Text = "4. Simple WinForms Application";
        this.Height = 100;
        this.Width = 300;
    }

    // handle 'OnPaint' event
    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // retrieve Graphics object to paint to the control
        Graphics g = e.Graphics;

        // create a pen
        Pen p = new Pen(Color.Black, 10);
        
        // draw a line from the last to the current position
        g.DrawLine(p, 20, 40, 280, 40);

        // release all resources
        p.Dispose ();
    }
}
#endif

#if WINFORMS_05
using System;
using System.Drawing;
using System.Windows.Forms;

class Demo
{
    public static void Main ()
    {
        MyForm f = new MyForm ();
        Application.Run (f);
    }
}

class MyForm : Form
{
    // default c'tor
    public MyForm ()
    {
        // set caption bar text and size of this form
        this.Text = "5. Simple WinForms Application";
        this.Height = 100;
        this.Width = 400;

        // create a menu
        MainMenu mainMenu = new MainMenu();
        MenuItem menu = new MenuItem ("Menu");
        MenuItem item = new MenuItem ("Item");
        menu.MenuItems.Add (item);
        mainMenu.MenuItems.Add (menu);
        this.Menu = mainMenu;

        // create an 'EventHandler' delegate
        EventHandler handler = new EventHandler (this.MenuClicked);

        // subscribe delegate to the menu's Click-event
        item.Click += handler;        
    }

    // handle 'Click' event of menu item
    private void MenuClicked (Object sender, EventArgs e)
    {
        // create a Graphics object for this form
        Graphics g = this.CreateGraphics ();

        // create a Font and a Brush object
        Font f = new Font ("Courier", 20);
        SolidBrush br = new SolidBrush (Color.Blue);

        // draw a simple text message
        DateTime dt = DateTime.Now;
        String s = String.Format ("Menu clicked at {0}",
            dt.ToLongTimeString ());
        g.DrawString (s, f, br, 80, 10);

        // release used resources
        f.Dispose ();
        br.Dispose ();
        g.Dispose ();
    }
}
#endif

#if WINFORMS_06
using System;
using System.Drawing;
using System.Windows.Forms;

class Demo
{
    public static void Main ()
    {
        MyForm f = new MyForm ();
        Application.Run (f);
    }
}

class MyForm : Form
{
    private String message;

    // default c'tor
    public MyForm ()
    {
        // set caption bar text and size of this form
        this.Text = "6. Simple WinForms Application";
        this.Height = 100;
        this.Width = 400;

        // create a menu
        MainMenu mainMenu = new MainMenu();
        MenuItem menu = new MenuItem ("Menu");
        MenuItem item = new MenuItem ("Item");
        item.Click += new EventHandler (this.MenuClicked);
        menu.MenuItems.Add (item);
        mainMenu.MenuItems.Add (menu);
        this.Menu = mainMenu;

        // no menu event yet available
        this.message = String.Empty;
    }

    // handle 'Click' event of menu item
    private void MenuClicked (Object sender, EventArgs e)
    {
        // store time of menu event
        DateTime dt = DateTime.Now;
        this.message = String.Format ("Menu clicked at {0}",
            dt.ToLongTimeString ());

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }

    // handle 'OnPaint' event
    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // draw message, if there is any
        if (this.message != String.Empty)
        {
            // retrieve Graphics object for this form
            Graphics g = e.Graphics;

            // create a Font and a Brush object
            Font f = new Font ("Courier", 20);
            SolidBrush br = new SolidBrush (Color.Blue);

            // draw a simple text message
            g.DrawString (this.message, f, br, 50, 10);

            // release all resources
            f.Dispose ();
            br.Dispose ();
        }
    }
}
#endif

#if WINFORMS_07
using System;
using System.Drawing;
using System.Windows.Forms;

class Demo
{
    public static void Main ()
    {
        MyForm f = new MyForm ();
        Application.Run (f);
    }
}

class MyForm : Form
{
    private String message;
    private Font f;
    private SolidBrush br;
    private bool disposed;

    // default c'tor
    public MyForm ()
    {
        // set caption bar text and size of this form
        this.Text = "7. Simple WinForms Application";
        this.Height = 100;
        this.Width = 400;

        // track whether 'Dispose' has been called
        this.disposed = false;

        // create a Font and a Brush object
        this.f = new Font ("Courier", 20);
        this.br = new SolidBrush (Color.Blue);

        // create main menu with event handler
        MainMenu mainMenu = new MainMenu();
        MenuItem menu = new MenuItem ("Menu");
        MenuItem item = new MenuItem ("Item");
        menu.MenuItems.Add (item);
        mainMenu.MenuItems.Add (menu);
        this.Menu = mainMenu;
        this.message = String.Empty;
        item.Click += new EventHandler (this.MenuClicked);
    }

    // handle 'Click' event of menu item
    private void MenuClicked (Object sender, EventArgs e)
    {
        // store time of menu event
        DateTime dt = DateTime.Now;
        this.message = String.Format
            ("Menu clicked at {0}", dt.ToLongTimeString ());

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }

    // handle 'OnPaint' event
    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // draw message, if there is any
        if (this.message != String.Empty)
        {
            // retrieve Graphics object for this form
            Graphics g = e.Graphics;

            // draw a simple text message
            g.DrawString (this.message, this.f, this.br, 50, 10);
        }
    }

    // release the resources used by this form
    protected override void Dispose (bool disposing)
    {
        // check to see if 'Dispose' has already been called
        if (! this.disposed)
        {
            // 'Dispose' has been called by a user's code
            if (disposing)
            {
                // release managed resources by this object
                this.f.Dispose ();
                this.br.Dispose ();
            }

            // set guard
            this.disposed = true;
        }

        // call 'Dispose' on base class
        base.Dispose (disposing);
    }
}
#endif
