#define WIRE_FRAME
// #define FILLED_POLYGON
// #define WITH_EVENTS

using System;
using System.Windows.Forms;
using System.Drawing;

#if WIRE_FRAME
class TreeOfPythagoras : Form
{
    private int limit;

    // default c'tor
    public TreeOfPythagoras ()
    {
        // set caption bar text and size of this form
        this.Text = "Tree Of Pythagoras";
        this.BackColor = Color.White;
        this.ClientSize = new Size (420, 300);

        // end of recursion
        this.limit = 4;
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // translate the coordinate system by a vector
        e.Graphics.TranslateTransform (0.0F, 300.0F);

        // start recursive drawing procedure
        Point p1 = new Point (240, -20);
        Point p2 = new Point (180, -20);

        this.DrawPythagoras (e.Graphics, p1, p2);
    }

    private void DrawPythagoras (Graphics g, Point p1, Point p2) 
    {
        // calculate difference vector
        int vx = p2.X - p1.X;
        int vy = p2.Y - p1.Y;

        // end of recursion reached ?
        if ( (vx * vx + vy * vy) < this.limit)
            return;

        // calculate missing points of square
        Point p3 = new Point (p1.X - vy, p1.Y + vx);
        Point p4 = new Point (p2.X - vy, p2.Y + vx);

        // draw square
        Point[] squarePoints = { p1, p2, p4, p3 };
        g.DrawPolygon (Pens.Black, squarePoints);

        // calculate difference vector of upper line
        vx = (p4.X - p3.X) / 2;
        vy = (p4.Y - p3.Y) / 2;

        // calculate missing point of isosceles triangle
        Point p5 = new Point (p3.X + vx - vy, p3.Y + vy + vx);

        // draw next two parts of the tree of Pythagoras
        this.DrawPythagoras (g, p3, p5);
        this.DrawPythagoras (g, p5, p4);
    }
}
#endif

#if FILLED_POLYGON
class TreeOfPythagoras : Form
{
    private int limit;
    private Random rand;

    // default c'tor
    public TreeOfPythagoras ()
    {
        // set caption bar text and size of this form
        this.Text = "Tree Of Pythagoras";
        this.BackColor = Color.White;
        this.ClientSize = new Size (420, 300);

        // end of recursion
        this.limit = 4;

        // need random numbers for coloring polygons
        this.rand = new Random ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // translate the coordinate system by a vector
        e.Graphics.TranslateTransform (0.0F, 300.0F);

        // start recursive drawing procedure
        Point p1 = new Point (240, -20);
        Point p2 = new Point (180, -20);

        this.DrawPythagoras (e.Graphics, p1, p2);
    }

    private void DrawPythagoras (Graphics g, Point p1, Point p2)
    {
        // calculate difference vector
        int vx = p2.X - p1.X;
        int vy = p2.Y - p1.Y;

        // end of recursion reached ?
        if ( (vx * vx + vy * vy) < this.limit)
            return;

        // calculate missing points of square
        Point p3 = new Point (p1.X - vy, p1.Y + vx);
        Point p4 = new Point (p2.X - vy, p2.Y + vx);

        // draw square
        Point[] squarePoints = { p1, p2, p4, p3 };
        int red = this.rand.Next (256);
        int green = this.rand.Next (256);
        int blue = this.rand.Next (256);
        Color c = Color.FromArgb (red, green, blue);
        Brush b = new SolidBrush (c);
        g.FillPolygon (b, squarePoints);
        b.Dispose ();

        // calculate difference vector of upper line
        vx = (p4.X - p3.X) / 2;
        vy = (p4.Y - p3.Y) / 2;

        // calculate missing point of isosceles triangle
        Point p5 = new Point (p3.X + vx - vy, p3.Y + vy + vx);

        // draw missing lines of triangle
        Point[] trianglePoints = { p3, p5, p4 };
        red = this.rand.Next (256);
        green = this.rand.Next (256);
        blue = this.rand.Next (256);
        c = Color.FromArgb (red, green, blue);
        b = new SolidBrush (c);
        g.FillPolygon (b, trianglePoints);
        b.Dispose ();

        // draw next two parts of the tree of Pythagoras
        this.DrawPythagoras (g, p3, p5);
        this.DrawPythagoras (g, p5, p4);
    }
}

#endif

#if WITH_EVENTS
class TreeOfPythagoras : Form
{
    private int limit;
    private int level;

    // default c'tor
    public TreeOfPythagoras ()
    {
        // set caption bar text and size of this form
        this.Text = "Tree Of Pythagoras";
        this.BackColor = Color.White;
        this.ClientSize = new Size (420, 300);

        // end of recursion
        this.limit = 4;
        this.level = 4;
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // translate the coordinate system by a vector
        e.Graphics.TranslateTransform (0.0F, 300.0F);

        // start recursive drawing procedure
        Point p1 = new Point (240, -20);
        Point p2 = new Point (180, -20);

        this.DrawPythagoras (e.Graphics, p1, p2, this.level);
    }

    private void DrawPythagoras (Graphics g, Point p1, Point p2, int level) 
    {
        // calculate difference vector
        int vx = p2.X - p1.X;
        int vy = p2.Y - p1.Y;

        // end of recursion reached ?
        if ( (vx * vx + vy * vy) < this.limit)
            return;

        // level reached ?
        level --;
        if (level < 0)
            return;

        // calculate missing points of square
        Point p3 = new Point (p1.X - vy, p1.Y + vx);
        Point p4 = new Point (p2.X - vy, p2.Y + vx);

        // draw square
        Point[] squarePoints = { p1, p2, p4, p3 };
        g.DrawPolygon (Pens.Black, squarePoints);

        // calculate difference vector of upper line
        vx = (p4.X - p3.X) / 2;
        vy = (p4.Y - p3.Y) / 2;

        // calculate missing point of isosceles triangle
        Point p5 = new Point (p3.X + vx - vy, p3.Y + vy + vx);

        // draw next two parts of the tree of Pythagoras
        this.DrawPythagoras (g, p3, p5, level);
        this.DrawPythagoras (g, p5, p4, level);
    }

    // handle 'MouseDown' events
    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        if (e.Button == MouseButtons.Left)
        {
            this.level ++;
        }
        else if (e.Button == MouseButtons.Right)
        {
            if (this.level > 0)
                this.level --;
        }

        String s = String.Format ("Level: {0}", this.level);
        this.Text = s;

        this.Invalidate ();
    }
}
#endif
