#define LISTING_01
// #define LISTING_02
// #define LISTING_03

using System;
using System.Drawing;
using System.Windows.Forms;

#if LISTING_01
class MouseClicksForm : Form
{
    public MouseClicksForm ()
    {
        this.Text = "MouseClicks";
        this.ClientSize = new Size (400, 100);
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // create font resource
        Font f = new Font ("Courier", 8);

        // simple output
        String s = String.Format ("{0}, {1}", e.X, e.Y);
        Graphics g = this.CreateGraphics ();
        g.DrawString (s, f, Brushes.Blue, e.X, e.Y);

        // release used resources
        f.Dispose ();
        g.Dispose ();
    }
}
#endif

#if LISTING_02
class MouseClicksForm : Form
{
    private int x;
    private int y;

    public MouseClicksForm ()
    {
        this.Text = "MouseClicks";
        this.ClientSize = new Size (400, 100);
        
        this.x = -1;
        this.y = -1;
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        if (this.x != -1)
        {
            // create font resource
            Font f = new Font ("Courier", 8);

            // simple output
            String s = String.Format ("{0}, {1}", this.x, this.y);
            Graphics g = e.Graphics;
            g.DrawString (s, f, Brushes.Blue, this.x, this.y);

            // release font resource
            f.Dispose ();
        }
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // update instance variables
        this.x = e.X;
        this.y = e.Y;

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }
}
#endif

#if LISTING_03

using System.Collections;

class MouseClicksForm : Form
{
    private ArrayList list;

    public MouseClicksForm ()
    {
        this.Text = "MouseClicks";
        this.ClientSize = new Size (400, 100);
        this.list = new ArrayList ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // draw points
        if (this.list.Count > 0)
        {
            Font f = new Font ("Courier", 8);
            for (int i = 0; i < this.list.Count; i += 2)
            {
                // retrieve coordinates
                int x = (int) this.list[i];
                int y = (int) this.list[i+1];

                // simple output
                String s = String.Format ("{0}, {1}", x, y);
                e.Graphics.DrawString (s, f, Brushes.Blue, x, y);
            }
            f.Dispose ();
        }
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // add point
        this.list.Add (e.X);
        this.list.Add (e.Y);

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }
}
#endif