// #define LISTING_01
#define LISTING_02
// #define LISTING_03

#define NET_1
// #define NET_2

using System;
using System.Windows.Forms;
using System.Drawing;

#if LISTING_01
class ScribbleForm : Form
{
    private int lastX;
    private int lastY;

    public ScribbleForm ()
    {        
        this.Text = "Scribble";
    }

    // handle 'MouseDown' event
    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // get first point of scribble
        this.lastX = e.X;
        this.lastY = e.Y;
    }
 
    // handle 'MouseMove' event
    protected override void OnMouseMove (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseMove' events
        base.OnMouseMove (e);

        MouseButtons b = e.Button;
        if (b == MouseButtons.Left)
        {
            // retrieve current coordinates
            int tmpX = e.X;
            int tmpY = e.Y;

            // create a pen
            Pen p = new Pen (Color.Black, 2);

            // retrieve a graphics object for this form
            Graphics g = this.CreateGraphics ();
            
            // draw a line from the last to the current position
            g.DrawLine (p, lastX, lastY, tmpX, tmpY);

            // save coordinates of current position
            this.lastX = tmpX;
            this.lastY = tmpY;

            // release used resources
            p.Dispose ();
            g.Dispose ();
        }
    }
}
#endif

#if LISTING_02
using System.Collections;

class ScribbleForm : Form
{
    private ArrayList list;

    public ScribbleForm ()
    {        
        this.Text = "Scribble";
        this.Size = new Size (400, 200);
        this.list = new ArrayList ();

        // enable double-buffering
#if NET_1
        this.SetStyle (
            ControlStyles.DoubleBuffer | 
            ControlStyles.UserPaint | 
            ControlStyles.AllPaintingInWmPaint,
            true
        );
#endif

#if NET_2
        this.DoubleBuffered = true;
#endif
    }

    // handle 'MouseDown' event
    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // store first point of scribble
        this.list.Add (e.X);
        this.list.Add (e.Y);
    }
 
    // handle 'MouseMove' event
    protected override void OnMouseMove (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseMove' events
        base.OnMouseMove (e);

        if (e.Button == MouseButtons.Left)
        {
            // store current coordinates
            this.list.Add (e.X);
            this.list.Add (e.Y);

            // initiate an 'OnPaint' method invocation
            this.Invalidate ();
        }
    }

    // handle 'MouseDown' event
    protected override void OnMouseUp (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseUp' events
        base.OnMouseUp (e);

        // get last point of scribble
        this.list.Add (e.X);
        this.list.Add (e.Y);

        // terminate last scribble
        this.list.Add (-1);
        this.list.Add (-1);

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        base.OnPaint (e);

        // create a pen
        Pen p = new Pen (Color.Black, 2);
        
        for (int i = 3; i < this.list.Count; i += 2) 
        {
            // retrieve two points to connect
            int x1 = (int) this.list[i-3];
            int y1 = (int) this.list[i-2];
            int x2 = (int) this.list[i-1];
            int y2 = (int) this.list[i];

            // end of a scribble reached?
            if ((x1 == -1 && y1 == -1) || (x2 == -1 && y2 == -1))
                continue;

            // draw a line from the last to the current point
            e.Graphics.DrawLine (p, x1, y1, x2, y2);
        }

        // release used resources
        p.Dispose ();
    }
}
#endif

#if LISTING_03
using System.Collections;

class ScribblePanel : Panel
{
    private ArrayList list;
    private Color penColor;

    public ScribblePanel ()
    {
        this.list = new ArrayList ();
        this.penColor = Color.Black;
        this.BorderStyle = BorderStyle.Fixed3D;

        // enable double-buffering
#if NET_1
        this.SetStyle (
            ControlStyles.DoubleBuffer | 
            ControlStyles.UserPaint | 
            ControlStyles.AllPaintingInWmPaint,
            true
        );
#endif

#if NET_2
        this.DoubleBuffered = true;
#endif
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // store first point of scribble
        this.list.Add (e.X);
        this.list.Add (e.Y);
    }
 
    protected override void OnMouseMove (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseMove' events
        base.OnMouseMove (e);

        if (e.Button == MouseButtons.Left)
        {
            // store current coordinates
            this.list.Add (e.X);
            this.list.Add (e.Y);

            // initiate an 'OnPaint' method invocation
            this.Invalidate ();
        }
    }

    protected override void OnMouseUp (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseUp' events
        base.OnMouseUp (e);

        // get last point of scribble
        this.list.Add (e.X);
        this.list.Add (e.Y);

        // terminate last scribble
        this.list.Add (-1);
        this.list.Add (-1);

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // create a pen
        Pen p = new Pen (this.penColor, 2);
        
        for (int i = 3; i < this.list.Count; i += 2) 
        {
            // retrieve two points to connect
            int x1 = (int) this.list[i-3];
            int y1 = (int) this.list[i-2];
            int x2 = (int) this.list[i-1];
            int y2 = (int) this.list[i];

            // end of a scribble reached?
            if ((x1 == -1 && y1 == -1) || (x2 == -1 && y2 == -1))
                continue;

            // draw a line from the last to the current point
            e.Graphics.DrawLine (p, x1, y1, x2, y2);
        }

        // release used resources
        p.Dispose ();
    }

    public Color PenColor
    {
        set
        {
            this.penColor = value;

            // initiate an 'OnPaint' method invocation
            this.Invalidate ();
        }

        get
        {
            return this.penColor;
        }
    }

    public void Clear ()
    {
        this.list.Clear ();
        this.Invalidate ();
    }
}
    
class ScribbleForm : Form
{
    private ScribblePanel p;
    private Button b;
    private ComboBox cb;

    public ScribbleForm ()
    {        
        // setup form
        this.Text = "Scribble";
        this.ClientSize = new Size (500, 250);

        // setup panel
        this.p = new ScribblePanel ();
        this.p.Location = new Point (10, 40);
        this.p.Size = new Size (
            this.ClientSize.Width - 20, this.ClientSize.Height - 50
        );
        this.p.Anchor =
            AnchorStyles.Left | AnchorStyles.Right |
            AnchorStyles.Bottom | AnchorStyles.Top;

        // setup button
        this.b = new Button ();
        this.b.Text = "Clear";
        this.b.Location = new Point (10, 10);
        this.b.Width = 190;
        this.b.Click += new EventHandler (this.OnButtonClick);

        // setup combo box
        this.cb = new ComboBox ();
        this.cb.Items.Add (Color.Black.Name);
        this.cb.Items.Add (Color.Blue.Name);
        this.cb.Items.Add (Color.Cyan.Name);
        this.cb.Items.Add (Color.DarkGray.Name);
        this.cb.Items.Add (Color.Gray.Name);
        this.cb.Items.Add (Color.Green.Name);
        this.cb.Items.Add (Color.LightGray.Name);
        this.cb.Items.Add (Color.Magenta.Name);
        this.cb.Items.Add (Color.Orange.Name);
        this.cb.Items.Add (Color.Pink.Name);
        this.cb.Items.Add (Color.Red.Name);
        this.cb.Items.Add (Color.White.Name);
        this.cb.Items.Add (Color.Yellow.Name);
        this.cb.SelectedIndex = 0;
        this.cb.Location = new Point (210, 10);
        this.cb.Width = 200;
        String s = (String) this.cb.SelectedItem;
        this.p.PenColor = Color.FromName (s);
        this.cb.SelectedIndexChanged +=
            new EventHandler (this.OnSelectedIndexChanged);

        // add controls to container
        this.Controls.Add (b);
        this.Controls.Add (this.p);
        this.Controls.Add (cb);
    }

    public void OnButtonClick (Object sender, EventArgs e)
    {
        this.p.Clear ();
    }

    public void OnSelectedIndexChanged (Object sender, EventArgs e)
    {
        String s = (String) this.cb.SelectedItem;
        this.p.PenColor = Color.FromName (s);
    }
}
#endif