#define LISTING_01
// #define LISTING_02
// #define LISTING_03
// #define LISTING_04

using System;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;

#if LISTING_01
// event source
class SimpleMouseEventsForm : Form
{
    // default c'tor
    public SimpleMouseEventsForm ()
    {
        // set caption bar text and size of this form
        this.Text = "Simple Mouse Events";
        this.ClientSize = new Size (400, 100);
    }
}

// event sink
class MouseEventSink
{
    public void OnMouseDown (Object sender, MouseEventArgs e)
    {
        String s = String.Format ("clicked at {0}, {1}", e.X, e.Y);
        MessageBox.Show (s, "Mouse Info");
    }
}

class Demo
{
    public static void Main ()
    {
        // create event source
        SimpleMouseEventsForm f = new SimpleMouseEventsForm ();
        
        // create and register event sink
        MouseEventSink sink = new MouseEventSink ();
        f.MouseDown += new MouseEventHandler (sink.OnMouseDown);

        Application.Run (f);
    }
}
#endif

#if LISTING_02
class SimpleMouseEventsForm : Form
{
    // default c'tor
    public SimpleMouseEventsForm ()
    {
        // set caption bar text and size of this form
        this.Text = "Simple Mouse Events";
        this.ClientSize = new Size (400, 100);

        // create and register event sink
        this.MouseDown += new MouseEventHandler (this.OnMouseDown);
    }

    private void OnMouseDown (Object sender, MouseEventArgs e)
    {
        String s = String.Format ("mouse clicked at {0}, {1}", e.X, e.Y);
        this.Text = s;
    }
}

class Demo
{
    public static void Main ()
    {
        SimpleMouseEventsForm f = new SimpleMouseEventsForm ();
        Application.Run (f);
    }
}
#endif

#if LISTING_03
class SimpleMouseEventsForm : Form
{
    // default c'tor
    public SimpleMouseEventsForm ()
    {
        // set caption bar text and size of this form
        this.Text = "Simple Mouse Events";
        this.ClientSize = new Size (400, 100);
    }

    // handle 'MouseDown' events
    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        String s = String.Format ("mouse clicked at {0}, {1}", e.X, e.Y);
        this.Text = s;
    }
}

class Demo
{
    public static void Main ()
    {
        SimpleMouseEventsForm f = new SimpleMouseEventsForm ();
        Application.Run (f);
    }
}
#endif

#if LISTING_04
class SimpleMouseEventsForm : Form
{
    private int x;
    private int y;
    private Font f;
    private MouseEventHandler handler;

    // default c'tor
    public SimpleMouseEventsForm ()
    {
        // set caption bar text and size of this form
        this.Text = "Correctly hooking up to an event ...";
        this.ClientSize = new Size (400, 100);

        // reset instance variables
        this.x = -1;
        this.y = -1;

        // create 'MouseEnter' event sink
        this.handler = new MouseEventHandler (this.OnMouseMove);

        // allocate resources
        this.f = new Font ("Courier", 12, FontStyle.Bold);
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        if (e.Button == MouseButtons.Left)
        {
            // add 'MouseEnter' event sink (only once)
            if (this.x == -1)
            {
                this.MouseMove += handler;
            }
        }
        else if (e.Button == MouseButtons.Right)
        {
            // remove 'MouseMove' event sink (if exists)
            if (this.x >= 0)
            {
                this.MouseMove -= handler;

                // clear background
                this.x = -1;
                this.Invalidate ();
            }
        }
    }

    private void OnMouseMove (Object sender, MouseEventArgs e)
    {
        this.x = e.X;
        this.y = e.Y;
        this.Invalidate ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        if (this.x >= 0)
        {
            String s = String.Format ("Mouse at {0}, {1}", this.x, this.y);
            e.Graphics.DrawString (s, this.f, Brushes.Black, 20, 10);
        }
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        // enable registered clients to receive 'Closing' events
        base.OnClosing (e);

        // release resources
        f.Dispose ();

        // remove 'MouseEnter' event sink
        this.MouseMove -= handler;
    }
}

class Demo
{
    public static void Main ()
    {
        SimpleMouseEventsForm f = new SimpleMouseEventsForm ();
        Application.Run (f);
    }
}
#endif