#define VARIANT_01
// #define VARIANT_02

using System;
using System.Drawing;
using System.Windows.Forms;

#if VARIANT_01
// text box excercise: PIN number (offensive)
class StandardWinForms : Form
{
    private Label l1;
    private TextBox tb;
    private Button b;
    private Label l2;

    // flag used to determine when a character
    // other than a number has been entered
    private bool nonNumberEntered;

    public StandardWinForms ()
    {
        // setup form
        this.Text = "Standard WinForms Controls: Text boxes";
        this.Size = new Size (350, 70);

        // setup label
        this.l1 = new Label ();
        this.l1.Text = "Enter PIN:";
        this.l1.TextAlign = ContentAlignment.MiddleRight;
        this.l1.BorderStyle = BorderStyle.Fixed3D;
        this.l1.Size = new Size (60, 20);
        this.l1.Location = new Point (10, 10);
        this.Controls.Add (this.l1);

        // setup text box
        this.tb = new TextBox ();
        this.tb.Multiline = false;
        this.tb.Text = "";
        this.tb.MaxLength = 10;
        this.tb.Select (0, 0);
        this.tb.TextAlign = HorizontalAlignment.Left;
        this.tb.Size = new Size (80, 20);
        this.tb.Location = new Point (80, 10);
        this.tb.KeyDown += new KeyEventHandler (this.TextBox_KeyDown);
        this.tb.KeyPress += new KeyPressEventHandler (this.TextBox_KeyPress);
        this.Controls.Add (this.tb);

        // setup button
        this.b = new Button ();
        this.b.Text = "Clear";
        this.b.Size = new Size (60, 20);
        this.b.Location = new Point (170, 10);
        this.b.Click += new EventHandler (this.Button_Click);
        this.Controls.Add (b);

        // setup second label
        this.l2 = new Label ();
        this.l2.Text = "";
        this.l2.TextAlign = ContentAlignment.MiddleLeft;
        this.l2.BorderStyle = BorderStyle.Fixed3D;
        this.l2.Size = new Size (80, 20);
        this.l2.Location = new Point (240, 10);
        this.Controls.Add (this.l2);
    }

    private void Button_Click (Object sender, EventArgs e)
    {
        this.tb.Clear ();
        this.l2.Text = "";
        this.tb.Focus ();
    }

private void TextBox_KeyDown (Object sender, KeyEventArgs e)
{
    // initialize the flag to false
    this.nonNumberEntered = false;

    // keystroke is a number from the top of the keyboard
    if (e.KeyCode < Keys.D0 || e.KeyCode > Keys.D9)
    {
        // keystroke is a number from the keypad
        if (e.KeyCode < Keys.NumPad0 || e.KeyCode > Keys.NumPad9)
        {
            // keystroke is a backspace or an enter
            if(e.KeyCode != Keys.Back && e.KeyCode != Keys.Enter)
            {
                // a non-numerical keystroke was pressed:
                // set flag and evaluate keystroke in 'KeyPress' event
                this.nonNumberEntered = true;
            }
        }
    }
}

private void TextBox_KeyPress (Object sender, KeyPressEventArgs e)
{
    // check for 'nonNumberEntered' flag being set in the KeyDown event
    if (this.nonNumberEntered)
    {
        // stop the character from being entered
        // into the control since it is non-numerical
        e.Handled = true;
    }
    else
    {
        // handle key
        if (e.KeyChar == '\r')
        {
            // end of input: copy pin number into label
            this.l2.Text = this.tb.Text;
        }
    }
}
}
#endif

#if VARIANT_02
// text box excercise: PIN number (defensive)
class StandardWinForms : Form
{
    private Label l1;
    private TextBox tb;
    private Button b;
    private Label l2;

    public StandardWinForms ()
    {
        this.Text = "Standard WinForms Controls: Text boxes";
        this.Size = new Size (350, 70);
        this.TextBoxDemo ();
    }

    private void TextBoxDemo ()
    {
        // setup label
        this.l1 = new Label ();
        this.l1.Text = "Enter PIN:";
        this.l1.TextAlign = ContentAlignment.MiddleRight;
        this.l1.BorderStyle = BorderStyle.Fixed3D;
        this.l1.Size = new Size (60, 20);
        this.l1.Location = new Point (10, 10);
        this.Controls.Add (this.l1);

        // setup text box
        this.tb = new TextBox ();
        this.tb.Multiline = false;
        this.tb.Text = "";
        this.tb.MaxLength = 10;
        this.tb.Select (0, 0);
        this.tb.TextAlign = HorizontalAlignment.Left;
        this.tb.Size = new Size (80, 20);
        this.tb.Location = new Point (80, 10);
        this.tb.KeyPress += new KeyPressEventHandler (this.TextBox_KeyPress);
        this.Controls.Add (this.tb);

        // setup button
        this.b = new Button ();
        this.b.Text = "Clear";
        this.b.Size = new Size (60, 20);
        this.b.Location = new Point (170, 10);
        this.b.Click += new EventHandler (this.Button_Click);
        this.Controls.Add (b);

        // setup second label
        this.l2 = new Label ();
        this.l2.Text = "";
        this.l2.TextAlign = ContentAlignment.MiddleLeft;
        this.l2.BorderStyle = BorderStyle.Fixed3D;
        this.l2.Size = new Size (80, 20);
        this.l2.Location = new Point (240, 10);
        this.Controls.Add (this.l2);
    }

    private void Button_Click (Object sender, EventArgs e)
    {
        this.tb.Clear ();
        this.l2.Text = "";
        this.tb.Focus ();
    }

    private void TextBox_KeyPress (Object sender, KeyPressEventArgs e)
    {
        // handle key
        if (e.KeyChar == '\r')
        {
            // check syntax of input
            try
            {
                int n = Int32.Parse (this.tb.Text);
            }
            catch (Exception)
            {
                // no valid pin number
                return;
            }

            // valid pin number, copy into label
            this.l2.Text = this.tb.Text;
        }
    }
}
#endif

