using System;
using System.Drawing;
using System.Windows.Forms;

class SimpleEditorForm : Form
{
    private ToolbarPanel tp;
    private TextBox tb;
    private StatusPanel sp;

    public SimpleEditorForm ()
    {
        // setup form
        this.Text = "Standard WinForms Controls: Simple Editor";
        this.Size = new Size (500, 300);

        // setup toolbar panel
        this.tp = new ToolbarPanel ();
        this.tp.Dock = DockStyle.Top;
        this.tp.ConnectColorHandler (new EventHandler (this.OnColorChanged));
        this.tp.ConnectFontHandler (new EventHandler (this.OnFontChanged));

        // setup textbox
        this.tb = new TextBox ();
        this.tb.Multiline = true;
        this.tb.SelectionLength = 0;
        this.tb.Font = new Font (this.tp.SelectedFont, 10, FontStyle.Regular);
        this.tb.ScrollBars = ScrollBars.Vertical;
        this.tb.Dock = DockStyle.Fill;
        this.tb.TextChanged += new EventHandler (this.OnTextChanged);

        // setup status panel
        this.sp = new StatusPanel ();
        this.sp.Dock = DockStyle.Bottom;
        this.sp.Lines = 0;
        this.sp.Chars = 0;

        // add controls to container
        this.Controls.Add (this.tb);
        this.Controls.Add (this.tp);
        this.Controls.Add (this.sp);
    }

    private void OnColorChanged (Object sender, EventArgs e)
    {
        String s = this.tp.SelectedColor;
        Color c = Color.FromName (s);
        try
        {
            if (this.tp.IsForeColor)
                this.tb.ForeColor = c;
            else 
                this.tb.BackColor = c;
        }
        catch {} // ignore new settings
    }

    private void OnFontChanged (Object sender, EventArgs e)
    {
        String name = this.tp.SelectedFont;
        FontFamily family = new FontFamily (name);
        if (family.IsStyleAvailable (FontStyle.Regular))
        {
            Font f = new Font (name, 10, FontStyle.Regular);
            this.tb.Font = f;
        }
    }

    private void OnTextChanged (Object sender, EventArgs e)
    {
        String[] lines = this.tb.Lines;
        int numLines = lines.Length;

        int numChars = 0;
        for (int i = 0; i < numLines; i ++)
            numChars += lines[i].Length;

        this.sp.Lines = numLines;
        this.sp.Chars = numChars;
    }
}
