using System;
using System.Drawing;
using System.Drawing.Text;
using System.Windows.Forms;

class ToolbarPanel : Panel
{
    private ComboBox cb1;
    private ComboBox cb2;
    private RadioButton rb1;
    private RadioButton rb2;

    public ToolbarPanel ()
    {
        // setup panel
        this.BorderStyle = BorderStyle.Fixed3D;
        this.Height = 70;

        // setup colors combo box
        this.cb1 = new ComboBox ();
        Type enumType = typeof (KnownColor);
        String[] colors = Enum.GetNames (enumType);
        for (int i = 0; i < colors.Length; i ++)
            this.cb1.Items.Add (colors[i]);
        this.cb1.DropDownStyle = ComboBoxStyle.DropDown;
        this.cb1.SelectedItem = "White";
        this.cb1.Width = 150;
        this.cb1.Location = new Point (10, 10);

        // setup fonts combo box
        this.cb2 = new ComboBox ();
        InstalledFontCollection fonts = new InstalledFontCollection ();
        FontFamily[] families = fonts.Families;
        for (int i = 0; i < families.Length; i ++)
            this.cb2.Items.Add (families[i].Name);
        this.cb2.DropDownStyle = ComboBoxStyle.DropDown;
        this.cb2.SelectedIndex = 0;
        this.cb2.Width = 150;
        this.cb2.Location = new Point (320, 10);

        // setup two radio buttons
        this.rb1 = new RadioButton ();
        this.rb2 = new RadioButton ();
        this.rb1.Text = "Background Color";
        this.rb2.Text = "Foreground Color";
        this.rb1.Width = 120;
        this.rb2.Width = 120;
        this.rb1.Location = new Point (180, 10);
        this.rb2.Location = new Point (180, 35);
        this.rb1.Checked = true;
        this.rb2.Checked = false;

        // add controls to container
        this.Controls.Add (this.cb1);
        this.Controls.Add (this.cb2);
        this.Controls.Add (this.rb1);
        this.Controls.Add (this.rb2);
    }

    public void ConnectColorHandler (EventHandler handler)
    {
        this.cb1.SelectedIndexChanged += handler;
    }

    public void ConnectFontHandler (EventHandler handler)
    {
        this.cb2.SelectedIndexChanged += handler;
    }
    
    public String SelectedColor
    {
        get
        {
            return (String) this.cb1.SelectedItem;
        }
    }

    public String SelectedFont
    {
        get
        {
            return (String) this.cb2.SelectedItem;
        }
    }

    public bool IsForeColor
    {
        get
        {
            return this.rb2.Checked;
        }
    }
}