using System;
using System.Drawing;
using System.Windows.Forms;

public class PieChartElement
{
    private String name;
    private float percent;
    private Color color;

    public PieChartElement (String name, float percent, Color color)
    {
        this.name = name;
        this.percent = percent;
        this.color = color;
    }

    public String Name
    {
        get
        {
            return this.name;
        }
    }

    public float Percent
    {
        get
        {
            return this.percent;
        }
    }

    public Color PieColour
    {
        get
        {
            return this.color;
        }
    }
}

class PieChartListBox : Control
{
    private PieChartElement[] elements;

    protected override Size DefaultSize
    {
        get
        {
            return new Size (200, 160);
        }
    }

    public PieChartElement[] Elements
    {
        set
        {
            this.elements = value;
            this.Invalidate ();
        }
    }
    
    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        if (this.elements != null)
        {
            Font f = new Font ("Verdana", 8, FontStyle.Bold);
            int x = 15;
            int y = 0;
            for (int i = 0; i < this.elements.Length; i ++)
            {
                Brush b = new SolidBrush (this.elements[i].PieColour);
                e.Graphics.FillRectangle (b, x, y + f.Height / 2 - 2, 7, 7);
                String s = String.Format ("{0}:  {1}%",
                    this.elements[i].Name, this.elements[i].Percent);
                e.Graphics.DrawString (s, f, Brushes.Black, x + 8, y);
                y += f.Height + 5;
                b.Dispose ();
            }
            f.Dispose ();
        }
    }
}

class PieChartControl : Control
{
    private PieChartElement[] elements;

    protected override Size DefaultSize
    {
        get
        {
            return new Size (220, 160);
        }
    }
    
    public PieChartElement[] Elements
    {
        set
        {
            this.elements = value;
            this.Invalidate ();
        }
    }
    
    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        if (this.elements != null)
        {
            float startAngle = 180.0F;
            float sweepAngle = 0.0F;
            for (int i = 0; i < this.elements.Length; i ++)
            {
                Brush b = new SolidBrush (this.elements[i].PieColour);
                startAngle += sweepAngle;
                sweepAngle = this.elements[i].Percent * 3.6F;
                int angle = (int) Math.Round (sweepAngle);
                e.Graphics.FillPie (
                    b, 0, 0, this.Width, this.Height, startAngle, sweepAngle
                );
                b.Dispose ();
            }
        }
    }
}

public class PieChart : UserControl
{
    private PieChartControl chart;
    private PieChartListBox list;
    private Label header;

    public PieChart ()
    {
        // setup piechart control
        this.chart = new PieChartControl ();
        this.chart.Location = new Point (230, 80);

        // setup control
        this.BackColor = Color.White;
        this.Width = this.chart.Width + 250;
        this.Height = this.chart.Height + 100;

        // setup piechart listbox
        this.list = new PieChartListBox ();
        this.list.Location = new Point (5, 80);

        // setup header label
        this.header = new Label ();
        this.header.Text = "";
        this.header.TextAlign = ContentAlignment.MiddleCenter;
        this.header.Font = new Font ("Tahoma", 12, FontStyle.Bold);
        this.header.BorderStyle = BorderStyle.Fixed3D;
        this.header.Location = new Point (10, 15);
        this.header.Size = new Size (this.Width - 20, 30);

        // add controls to container
        this.Controls.Add (this.header);
        this.Controls.Add (this.list);
        this.Controls.Add (this.chart);
    }

    public String Title
    {
        set
        {
            this.header.Text = value;
        }
    }
    
    public PieChartElement[] Elements
    {
        set
        {
            this.chart.Elements = value;
            this.list.Elements = value;
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        base.OnPaint (e);

        e.Graphics.DrawRectangle (Pens.Black, 0, 0, this.Width - 1, this.Height - 1);
        e.Graphics.DrawRectangle (Pens.Black, 2, 2, this.Width - 5, this.Height - 5);
    }
}
