using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

public class BezierForm : Form
{
    private BezierPanel pBezier;
    private Button bClear;
    private ComboBox cbMode;
    private Label lXCoord;
    private Label lYCoord;
    private Label lResolution;
    private Label lConstructionParam;
    private TrackBar tbResolution;
    private TrackBar tbConstruction;
    private CheckBox cbShowControlPoints;
    private CheckBox cbBezierCurve;
    private CheckBox cbConstruction;

    public BezierForm ()
    {
        // setup form
        this.ClientSize = new Size (700, 400);
        this.Text = "Bezier Splines";

        // setup Bezier panel
        this.pBezier = new BezierPanel ();
        this.pBezier.MouseMove += new MouseEventHandler (this.BezierPanel_MouseMoved);
        this.pBezier.Size = new Size (650, 300);
        this.pBezier.Location = new Point (10, 50);

        // setup combo box
        this.cbMode = new ComboBox ();
        this.cbMode.Items.Add (BezierMode.Create);
        this.cbMode.Items.Add (BezierMode.Edit);
        this.cbMode.Items.Add (BezierMode.Delete);
        this.cbMode.DropDownStyle = ComboBoxStyle.DropDownList;
        this.cbMode.SelectedIndex = 0;
        this.cbMode.SelectedIndexChanged += new EventHandler (this.ComboBox_SelectedIndexChanged);
        this.cbMode.Width = 80;
        this.cbMode.Location = new Point (10, 10);

        // setup clear button
        this.bClear = new Button ();
        this.bClear.Text = "Clear";
        this.bClear.Click += new EventHandler (this.Button_Click);
        this.bClear.Width = 80;
        this.bClear.Location = new Point (100, 10);

        // setup two labels for current mouse coordinates
        this.lXCoord = new Label ();
        this.lYCoord = new Label ();
        this.lXCoord.TextAlign = ContentAlignment.MiddleCenter;
        this.lXCoord.BorderStyle = BorderStyle.Fixed3D;
        this.lXCoord.Width = 40;
        this.lXCoord.Location = new Point (190, 10);
        this.lYCoord.TextAlign = ContentAlignment.MiddleCenter;
        this.lYCoord.BorderStyle = BorderStyle.Fixed3D;
        this.lYCoord.Width = 40;
        this.lYCoord.Location = new Point (232, 10);

        // setup another label, just for static text
        Label lResolutionText = new Label ();
        lResolutionText.TextAlign = ContentAlignment.MiddleLeft;
        lResolutionText.Text = "Resolution:";
        lResolutionText.Width = 65;
        lResolutionText.Location = new Point (280, 10);

        // setup label for current resolution
        this.lResolution = new Label ();
        this.lResolution.TextAlign = ContentAlignment.MiddleCenter;
        this.lResolution.BorderStyle = BorderStyle.Fixed3D;
        this.lResolution.Width = 40;
        this.lResolution.Location = new Point (350, 10);

        // setup resolution track bar
        this.tbResolution = new TrackBar ();
        this.tbResolution.Minimum = 1;
        this.tbResolution.Maximum = 100;
        this.tbResolution.Value = 50;
        this.tbResolution.LargeChange = 1;
        this.tbResolution.SmallChange = 1;
        this.tbResolution.TickFrequency = 5;
        this.tbResolution.TickStyle = TickStyle.BottomRight;
        this.tbResolution.ValueChanged += new EventHandler (this.TrackBar_ValueChanged);
        this.tbResolution.Width = 250;
        this.tbResolution.Location = new Point (400, 7);

        // setup check boxes
        this.cbShowControlPoints = new CheckBox ();
        this.cbShowControlPoints.Text = "Show Control Points";
        this.cbShowControlPoints.Checked = true;
        this.cbShowControlPoints.CheckedChanged += new EventHandler (this.CheckBox_CheckedChanged);
        this.cbShowControlPoints.Width = 130;
        this.cbShowControlPoints.Location = new Point (10, 360);

        this.cbBezierCurve = new CheckBox ();
        this.cbBezierCurve.Text = "Show Bezier Curve";
        this.cbBezierCurve.Checked = true;
        this.cbBezierCurve.CheckedChanged += new EventHandler (this.CheckBox_CheckedChanged);
        this.cbBezierCurve.Width = 130;
        this.cbBezierCurve.Location = new Point (140, 360);

        this.cbConstruction = new CheckBox ();
        this.cbConstruction.Text = "Show Construction";
        this.cbConstruction.Checked = false;
        this.cbConstruction.CheckedChanged += new EventHandler (this.CheckBox_CheckedChanged);
        this.cbConstruction.Width = 130;
        this.cbConstruction.Location = new Point (270, 360);

        // setup another label, just for static text
        Label lConstructionParamText = new Label ();
        lConstructionParamText.TextAlign = ContentAlignment.MiddleLeft;
        lConstructionParamText.Text = "t:";
        lConstructionParamText.Width = 10;
        lConstructionParamText.Location = new Point (397, 360);

        // setup another label for current construction parameter
        this.lConstructionParam = new Label ();
        this.lConstructionParam.TextAlign = ContentAlignment.MiddleCenter;
        this.lConstructionParam.BorderStyle = BorderStyle.Fixed3D;
        this.lConstructionParam.Width = 50;
        this.lConstructionParam.Location = new Point (410, 360);

        // setup construction track bar
        this.tbConstruction = new TrackBar ();
        this.tbConstruction.Minimum = 0;
        this.tbConstruction.Maximum = 100;
        this.tbConstruction.Value = 50;
        this.tbConstruction.LargeChange = 1;
        this.tbConstruction.SmallChange = 1;
        this.tbConstruction.TickFrequency = 5;
        this.tbConstruction.TickStyle = TickStyle.BottomRight;
        this.tbConstruction.ValueChanged += new EventHandler (this.TrackBar_ValueChanged);
        this.tbConstruction.Width = 200;
        this.tbConstruction.Location = new Point (460, 360);
   
        // add controls to container
        this.Controls.Add (this.pBezier);
        this.Controls.Add (this.bClear);
        this.Controls.Add (this.cbMode);
        this.Controls.Add (this.lXCoord);
        this.Controls.Add (this.lYCoord);
        this.Controls.Add (this.lConstructionParam);
        this.Controls.Add (lResolutionText);
        this.Controls.Add (lResolution);
        this.Controls.Add (lConstructionParamText);
        this.Controls.Add (this.tbResolution);
        this.Controls.Add (this.cbShowControlPoints);
        this.Controls.Add (this.cbBezierCurve);
        this.Controls.Add (this.cbConstruction);
        this.Controls.Add (this.tbConstruction);
    }

    private void Button_Click (Object sender, EventArgs e)
    {
        this.pBezier.ClearControlPoints ();
    }

    private void ComboBox_SelectedIndexChanged (Object sender, EventArgs e)
    {
        BezierMode mode = (BezierMode) this.cbMode.SelectedItem;
        this.pBezier.Mode = mode;
    }

    private void TrackBar_ValueChanged (Object sender, EventArgs e)
    {
        if (sender == this.tbResolution)
        {
            this.pBezier.Resolution = this.tbResolution.Value;
            this.lResolution.Text = this.tbResolution.Value.ToString ();
        }
        else if (sender == this.tbConstruction)
        {
            // retrieve current value of scrollbar
            double value = (((double) this.tbConstruction.Value) / ((double) 100));
            this.lConstructionParam.Text = value.ToString ();
            this.pBezier.ConstructionPosition = value;
        }
    }

    private void CheckBox_CheckedChanged (Object sender, EventArgs e)
    {
        if (sender == this.cbShowControlPoints)
        {
            this.pBezier.ShowControlPoints = this.cbShowControlPoints.Checked;
        }
        else if (sender == this.cbBezierCurve)
        {
            this.pBezier.ShowBezierCurve = this.cbBezierCurve.Checked;
        }
        else if (sender == this.cbConstruction)
        {
            this.pBezier.ShowConstruction = this.cbConstruction.Checked;
        }
    }

    private void BezierPanel_MouseMoved (Object sender, MouseEventArgs e)
    {
        this.lXCoord.Text = e.X.ToString ();
        this.lYCoord.Text = e.Y.ToString ();
    }
}

class Demo
{
    public static void Main ()
    {
        BezierForm form = new BezierForm ();
        Application.Run (form);
    }
}
