using System;
using System.Drawing;
using System.Windows.Forms;

public class BezierForm : Form
{
    public BezierForm ()
    {
        // setup form
        this.ClientSize = new Size (792, 400);
        this.Text = "Bezier Splines";

        // create model
        BezierModelImpl model = new BezierModelImpl ();
        model.EnableControlPoints = true;
        model.EnableCurvePoints = true;
        model.EnableConstructionPoints = true;

        // create ui delegate (being connected to the model)
        BezierDelegate view = new BezierDelegate (model);
        view.Size = new Size (774, 300);
        view.Location = new Point (10, 45);
        
        // create textbox view (being connected to the model)
        BezierTextView textView = new BezierTextView (model);
        textView.Show ();

        // create button controller (being connected to the model)
        ClearController bClear = new ClearController (model);
        bClear.Location = new Point (10, 9);

        // create bezier mode controller (being connected to the view)
        BezierModeController modeController =
            new BezierModeController (view);
        modeController.Location = new Point (100, 10);

        // create position view (being connected to the ui delegate)
        PositionView positionView = new PositionView (view);
        positionView.Location = new Point (190, 10);

        // create bezier display controller (being connected to the model)
        BezierDisplayController displayController =
            new BezierDisplayController (model);
        displayController.Location = new Point (335, 4);

        // create resolution controller (being connected to the model)
        ResolutionController pResolution =
            new ResolutionController (model);
        pResolution.Location = new Point (10, 354);

        // create construction position controller
        // (being connected to the model)
        ConstructionPositionController pConstruction =
            new ConstructionPositionController (model);
        pConstruction.Location = new Point (400, 354);

        // add controls to the container
        this.Controls.Add (view);
        this.Controls.Add (positionView);
        this.Controls.Add (bClear);
        this.Controls.Add (pResolution);
        this.Controls.Add (pConstruction);
        this.Controls.Add (modeController);
        this.Controls.Add (displayController);
    }
}

