using System;
using System.Drawing;
using System.Windows.Forms;

public class BezierTextView : Form
{
    private BezierModel model;
    private TextBox tbView;

    public BezierTextView (BezierModel model)
    {
        this.model = model;

        // connect user interface delegate with model
        this.model.ControlPoints +=
            new ControlPointsEventHandler (this.ControlPointsChanged);

        // setup form
        this.ClientSize = new Size (250, 250);
        this.Text = "Bezier Text View";

        // setup text box
        this.tbView = new TextBox ();
        this.tbView.Multiline = true;
        this.tbView.ScrollBars = ScrollBars.Vertical;
        this.tbView.Dock = DockStyle.Fill;
        this.tbView.Font = new Font ("Courier", 10.0F, FontStyle.Bold);

        // add controls to the container
        this.Controls.Add (this.tbView);
    }

    private void ControlPointsChanged (BezierPoints controlPoints)
    {
        this.tbView.Clear ();

        if (controlPoints == null)
            return;
        
        for (int i = 0; i < controlPoints.Count; i ++)
        {
            BezierPoint p = (BezierPoint) controlPoints[i];
            String s = String.Format ("{0}: [{1},{2}]", i, p.X, p.Y);
            this.tbView.AppendText (s + "\r\n");
        }
    }

    protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
    {
        base.OnClosing (e);

        this.tbView.Font.Dispose ();
    }
}
