using System;
using System.Drawing;
using System.Windows.Forms;

class GameOfLifeForm : Form
{
    private GameOfLifePanel life;
    private Button b1;
    private Button b2;
    private ComboBox cb;

    private String[] patterns = new String[]
    {
        "Glider",
        "Small Exploder",
        "Large Exploder",
        "10 Cell Row",
        "Fish",
        "Pump",
        "Shooter"
    };

    public GameOfLifeForm ()
    {
        // setup form
        this.ClientSize = new Size (550, 420);
        this.Text = "Game of Life";
        this.BackColor = Color.LightGray;

        // setup right cell space panel
        this.life = new GameOfLifePanel ();
        this.life.Location = new Point (10, 10);
        this.Controls.Add (this.life);

        // setup 'Next' button
        this.b1 = new Button ();
        this.b1.Text = "Next";
        this.b1.Location = new Point (420, 10);
        this.b1.Click += new EventHandler (this.NextButton_Click);
        this.Controls.Add (this.b1);

        // setup 'Clear' button
        this.b2 = new Button ();
        this.b2.Text = "Clear";
        this.b2.Location = new Point (420, 40);
        this.b2.Click += new EventHandler (this.ClearButton_Click);
        this.Controls.Add (this.b2);

        // setup 'Standard Patterns' combo box
        this.cb = new ComboBox ();
        this.cb.Text = "Some Patterns";
        this.cb.Location = new Point (420, 70);
        for (int i = 0; i < this.patterns.Length; i ++)
            this.cb.Items.Add (this.patterns[i]);
        this.cb.SelectedIndexChanged +=
            new EventHandler (this.ComboBox_SelectedIndexChanged);
        this.Controls.Add (this.cb);
    }

    private void NextButton_Click (Object sender, EventArgs e)
    {
        this.life.Next ();
    }

    private void ClearButton_Click (Object sender, EventArgs e)
    {
        this.life.Clear ();
    }

    private void ComboBox_SelectedIndexChanged (Object sender, EventArgs e)
    {
        String item = (String) this.cb.SelectedItem;

        if (item.Equals (this.patterns[0]))  // glider
        {
            int[] shape = { 0,1, 1,2, 2,2, 2,1, 2,0 };
            this.life.DrawShape (shape, 3, 3);
        }
        else if (item.Equals (this.patterns[1])) // small exploder
        {
            int[] shape = { 0,1, 0,2, 1,0, 1,1, 1,3, 2,1, 2,2 };
            this.life.DrawShape (shape, 3, 4);
        }
        else if (item.Equals (this.patterns[2])) // large exploder
        {
            int[] shape = { 0,0, 0,1, 0,2, 0,3, 0,4, 2,0, 2,4,
                            4,0, 4,1, 4,2, 4,3, 4,4
                          };
            this.life.DrawShape (shape, 5, 5);
        }
        else if (item.Equals (this.patterns[3])) // 10 cell row
        {
            int[] shape = { 0,0, 1,0, 2,0, 3,0, 4,0, 5,0,
                            6,0, 7,0, 8,0, 9,0
                          };
            this.life.DrawShape (shape, 10, 1);
        }
        else if (item.Equals (this.patterns[4])) // fish
        {
            int[] shape = { 0,1, 0,3, 1,0, 2,0, 3,0, 3,3,
                            4,0, 4,1, 4,2
                          };
            this.life.DrawShape (shape, 5, 4);
        }            
        else if (item.Equals (this.patterns[5])) // pump
        {
            int[] shape = { 0,3, 0,4, 0,5, 1,0, 1,1, 1,5, 2,0, 2,1, 2,2,
                            2,3, 2,4, 4,0, 4,1, 4,2, 4,3, 4,4, 5,0, 5,1,
                            5,5, 6,3, 6,4, 6,5
                          };
            this.life.DrawShape (shape, 7, 6);
        }            
        else if (item.Equals (this.patterns[6])) // shooter
        {
            int[] shape = { 0,2, 0,3, 1,2, 1,3, 8,3, 8,4, 9,2, 9,4, 10,2,
                            10,3, 16,4, 16,5, 16,6, 17,4, 18,5, 22,1, 22,2,
                            23,0, 23,2, 24,0, 24,1, 24,12, 24,13, 25,12,
                            25,14, 26,12, 34,0, 34,1, 35,0, 35,1, 35,7,
                            35,8, 35,9, 36,7, 37,8 
                          };
            this.life.DrawShape (shape, 38, 15);
        }
    }
}
