using System;
using System.Drawing;
using System.Windows.Forms;

using CellularAutomataControls;

class CellularAutomataForm : Form
{
    private Button buttonClear;
    private Button buttonStart;
    private NumericUpDown numericUpDown;
    private RulePanel rulePanel;
    private AutomataPanel panel;

    public CellularAutomataForm ()
    {
        // main form
        this.ClientSize = new Size (634, 536);
        this.FormBorderStyle = FormBorderStyle.FixedDialog;
        this.MaximizeBox = false;
        this.MinimizeBox = false;
        this.Text = "Cellular Automata";

        // 'clear' button
        this.buttonClear = new Button ();
        this.buttonClear.Location = new Point (528, 8);
        this.buttonClear.Size = new Size (96, 23);
        this.buttonClear.Text = "Clear";
        EventHandler handler = new EventHandler (this.ClearButtonClick);
        this.buttonClear.Click += handler;
        this.Controls.Add (this.buttonClear);

        // 'start' button
        this.buttonStart = new Button ();
        this.buttonStart.Location = new Point (528, 48);
        this.buttonStart.Size = new Size (96, 23);
        this.buttonStart.TabIndex = 2;
        this.buttonStart.Text = "Start";
        EventHandler handler2 = new EventHandler (this.StartButtonClick);
        this.buttonStart.Click += handler2;
        this.Controls.Add (this.buttonStart);
    
        // numeric up-down control
        this.numericUpDown = new NumericUpDown ();
        this.numericUpDown.Location = new Point (64, 88);
        this.numericUpDown.Maximum = 255;
        this.numericUpDown.Size = new Size (128, 20);
        EventHandler handler3 =
            new EventHandler (this.NumericUpDownValueChanged);
        this.numericUpDown.ValueChanged += handler3;
        this.Controls.Add (this.numericUpDown);

        // label
        Label label1 = new Label ();
        label1.Location = new Point (8, 88);
        label1.Size = new Size (56, 23);
        label1.Text = "Rule:";
        label1.TextAlign = ContentAlignment.MiddleCenter;
        this.Controls.Add (label1);

        // rule panel
        this.rulePanel = new RulePanel ();
        this.rulePanel.Location = new Point (8,8);
        CodeNumberEventHandler handler4 =
            new CodeNumberEventHandler (this.RulePanelCodeNumberChanged);
        this.rulePanel.CodeNumberChanged += handler4;
        this.numericUpDown.Value = this.rulePanel.CodeNumber;
        this.Controls.Add (this.rulePanel);

        // automata panel
        this.panel = new AutomataPanel (this.rulePanel);
        this.panel.Location = new Point (8, 120);
        this.panel.Size = new Size (616, 408);
        this.Controls.Add (this.panel);
    }

    private void ClearButtonClick (Object sender, EventArgs e)
    {
        this.panel.Clear ();
    }

    private void StartButtonClick (Object sender, EventArgs e)
    {
        this.panel.Clear ();

        this.buttonClear.Enabled = false;
        this.buttonStart.Enabled = false;
        this.panel.Calculate ();
        this.buttonClear.Enabled = true;
        this.buttonStart.Enabled = true;

        this.panel.Invalidate ();
    }

    private void NumericUpDownValueChanged (Object sender, EventArgs e)
    {
        this.rulePanel.CodeNumber = (int) this.numericUpDown.Value;
    }

    protected void RulePanelCodeNumberChanged (int number)
    {
        this.numericUpDown.Value = this.rulePanel.CodeNumber;
    }
}

class Demo
{
    public static void Main () 
    {
        CellularAutomataForm f = new CellularAutomataForm ();
        Application.Run (f);
    }
}
