using System;
using System.Drawing;
using System.Windows.Forms;

namespace CellularAutomataControls
{
    public class AutomataPanel : Panel
    {
        private int width;
        private int height;

        private bool[,] cells;
        private Bitmap image;
        private Bitmap empty;
        private bool active;

        private RulePanel panel;

        private bool[] rules;

        public AutomataPanel (RulePanel panel)
        {
            this.panel = panel;
            this.image = null;
            this.empty = null;
            this.active = false;

            this.width = 0;
            this.height = 0;

            this.BorderStyle = BorderStyle.Fixed3D;
            this.BackColor = Color.White;
        }

        protected override void OnPaint (PaintEventArgs e)
        {
            // take care of registered delegates
            base.OnPaint (e);

            Graphics g = e.Graphics;
            g.DrawImage (this.active ? this.image : this.empty, 0, 0);
        }

        protected override void OnResize (EventArgs e)
        {
            // take care of registered delegates
            base.OnResize (e);

            this.width = this.Width;
            this.height = this.Height;

            this.cells = new bool [this.height, this.width];
            this.image = new Bitmap (this.width, this.height);

            // create empty bitmap
            this.empty = new Bitmap (this.width, this.height);

            this.active = false;
        }

        public void Calculate ()
        {
            // retrieve current automata rules
            this.rules = new bool[]
            {
                panel[0], panel[1], panel[2], panel[3],
                panel[4], panel[5], panel[6], panel[7]
            };
            
            // initialize first row
            for (int c = 0; c < this.width; c ++)
            {
                this.cells[0, c] = false;
                this.image.SetPixel (c, 0, Color.White);
            }

            // set pixel in the middle of the row
            this.cells[0, width / 2] = true;
            this.image.SetPixel (width / 2, 0, Color.Black);

            // calculate remaining rows
            for (int r = 1; r < this.height; r ++)
            {
                for (int c = 0; c < this.width; c ++)
                {
                    if (c == 0 || c == this.width - 1) // left or right border
                    {                        
                        this.image.SetPixel (c, r, Color.White);
                        continue;
                    }
                    else if (r == this.height - 1) // lower border
                    {
                        this.image.SetPixel (c, r, Color.White);
                        continue;
                    }
                    else
                    {
                        bool b0 = this.cells[r - 1, c + 1];
                        bool b1 = this.cells[r - 1, c];
                        bool b2 = this.cells[r - 1, c - 1];

                        int index = 0;
                        if (b0) index |= 0x01;
                        if (b1) index |= 0x02;
                        if (b2) index |= 0x04;
                        this.cells[r, c] = this.rules[index];
                        this.image.SetPixel (
                            c, r, this.cells[r, c] ? Color.Black : Color.White
                        );
                    }
                }
            }

            this.active = true;
        }

        public void Clear ()
        {
            this.active = false;
            this.Invalidate ();
        }
    }
}
