using System;
using System.Drawing;
using System.Windows.Forms;

namespace CellularAutomataControls
{
    public delegate void RuleStateEventHandler (int pos, bool state);

    public class Rule : Control
    {
        private bool state;
        private int position;

        private bool left;
        private bool middle;
        private bool right;

        private Rectangle r1;
        private Rectangle r2;
        private Rectangle r3;
        private Rectangle r4;

        // fired if rule's state has changed
        public event RuleStateEventHandler RuleStateChanged;

        public Rule ()
        {
            this.Size = new Size (49, 33);

            this.RuleStateChanged = null;

            this.left = false;
            this.middle = false;
            this.right = false;

            this.state = false;

            this.r1 = new Rectangle (0,  0,  16, 16);
            this.r2 = new Rectangle (16, 0,  16, 16);
            this.r3 = new Rectangle (32, 0,  16, 16);
            this.r4 = new Rectangle (16, 16, 16, 16);
        }
        
        protected override void OnPaint (PaintEventArgs e)
        {
            // take care of registered delegates
            base.OnPaint (e);

            Graphics g = e.Graphics;
            g.FillRectangle (this.left ? Brushes.Black : Brushes.White, this.r1);
            g.FillRectangle (this.middle ? Brushes.Black : Brushes.White, this.r2);
            g.FillRectangle (this.right ? Brushes.Black : Brushes.White, this.r3);
            g.FillRectangle (this.state ? Brushes.Black : Brushes.White, this.r4);

            g.DrawRectangle (Pens.Black, this.r1);
            g.DrawRectangle (Pens.Black, this.r2);
            g.DrawRectangle (Pens.Black, this.r3);
            g.DrawRectangle (Pens.Black, this.r4);
        }

        protected override void OnMouseUp (MouseEventArgs e)
        {
            // take care of registered delegates
            base.OnMouseUp (e);

            if (this.r4.Contains (e.X, e.Y))
            {
                this.state = ! this.state;
                this.Invalidate ();
                this.OnRuleStateChanged ();
            }
        }

        // rule state property
        public bool RuleState
        {
            get
            {
                return this.state;
            }

            set
            {
                if (this.state != value)
                {
                    this.state = value;
                    this.Invalidate ();

                    this.OnRuleStateChanged ();
                }
            }
        }

        // rule position property: must be in the range [0..7]
        public int RulePosition
        {
            get
            {
                return this.position;
            }

            set
            {
                if (value < 0 || value >= 8)
                {
                    String s = String.Format ("Wrong rule position: {0}", value);
                    ArgumentOutOfRangeException e = 
                        new ArgumentOutOfRangeException (s);
                    throw (e);
                }

                this.position = value;

                this.right =  ((value & 0x01) != 0) ? true : false;
                this.middle = ((value & 0x02) != 0) ? true : false;
                this.left =   ((value & 0x04) != 0) ? true : false;

                this.Invalidate ();
            }
        }
            
        protected virtual void OnRuleStateChanged ()
        {
            if (this.RuleStateChanged != null)
                this.RuleStateChanged (this.position, this.state);
        }
    }
}
