#define SYNCHRONOUS_VARIATION
// #define ASYNCHRONOUS_VARIATION_1
// #define ASYNCHRONOUS_VARIATION_2

using System;
using System.Threading;
using System.Runtime.Remoting.Messaging;

class PrimeFactorizer
{
    public static bool Factorize (long number, out long factor1, out long factor2)
    {
        factor1 = 1;
        factor2 = number;

        // write footprint of current thread to console
        int hashCode = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("[Factorize is running on thread {0}]", hashCode);

        // factorize number using a very simple approach
        for (long i = 2; i < number; i++)
        {
            if ((number % i) == 0)
            {
                factor1 = i;
                factor2 = number / i;
                return true;
            }
        }

        return false;
    }
}

#if SYNCHRONOUS_VARIATION
public class SynchronousDemo
{
    public static void Main ()
    {
        // write footprint of current thread to console
        int hashCode = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("[main thread is running on thread {0}]", hashCode);

        // invoke Factorizer synchronously
        Console.WriteLine ("invoking Factorizer synchronous ...");

        long number = ((long) 13821503) * ((long) 13821503);
        long factor1;
        long factor2;
        bool ret = PrimeFactorizer.Factorize (
            number, out factor1, out factor2
        );

        if (ret)
        {
            Console.WriteLine ("found factors {0} and {1}.",
                factor1, factor2);
        }
        else
        {
            Console.WriteLine ("{0} is prime.", number);
        }
    }
}
#endif

#if ASYNCHRONOUS_VARIATION_1
public class AsynchronousDemo
{
    // define a delegate for the Factorize method 
    public delegate bool FactorizeHandler (
        long number, 
        out long factor1,
        out long factor2
    );

    public static void Main ()
    {
        // write footprint of current thread to console
        int hashCode = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("[main thread is running on thread {0}]", hashCode);

        // create a delegate on the Factorize method of the Factorizer
        FactorizeHandler fd = new FactorizeHandler (PrimeFactorizer.Factorize);

        // asynchronously invoke the Factorize method
        Console.WriteLine (
            "invoking Factorizer asynchronously (variation 1) ..."
        );

        long number = ((long) 13821503) * ((long) 13821503);
        long temp;
        IAsyncResult ar = fd.BeginInvoke (
            number,      // 1. parameter of method Factorize
            out temp,    // 2. parameter of method Factorize
            out temp,    // 3. parameter of method Factorize
            null,        // AsyncCallback delegate object (not used)
            null         // user supplied object (not used)
        );

        // wait for completion of asynchronouos invocation
        while (! ar.IsCompleted)
        {
            Thread.Sleep (100); // sleep a couple of milliseconds
            Console.WriteLine ("waiting for results ...");
        }

        // obtain results
        long factor1, factor2; 
        bool ret = fd.EndInvoke (out factor1, out factor2, ar);

        // output results
        if (ret)
            Console.WriteLine ("found factors {0} and {1}.",
                factor1, factor2);
        else
            Console.WriteLine ("{0} is prime.", number);
    }
}
#endif

#if ASYNCHRONOUS_VARIATION_2

// define a delegate for the Factorize method 
public delegate bool FactorizeHandler (
    long number, 
    out long factor1,
    out long factor2
);

public class AsynchronousDemo
{
    public static void Main ()
    {
        // write footprint of current thread to console
        int hashCode = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("[main thread is running on thread {0}]", hashCode);

        // create a delegate on the Factorize method of the Factorizer
        FactorizeHandler fd =
			new FactorizeHandler (PrimeFactorizer.Factorize);

        // create an instance of the class which is going
        // to be called when the call completes
        FactorizeCompletion fc = new FactorizeCompletion ();

        // define the AsyncCallback delegate
        AsyncCallback cb = new AsyncCallback (fc.FactorizedResults);

        // asynchronously invoke the Factorize method on object pf
        Console.WriteLine (
            "invoking Factorizer asynchronously (variation 2) ..."
        );

        long number = ((long) 13821503) * ((long) 13821503);
        long temp;
        IAsyncResult ar = fd.BeginInvoke (
            number,       // 1. parameter of method Factorize
            out temp,     // 2. parameter of method Factorize
            out temp,     // 3. parameter of method Factorize
            cb,           // AsyncCallback delegate object
            null          // user supplied object - not used
        );

        // doing some other useful work
        // .............
        Console.WriteLine ("press enter to exit  ...");
        Console.ReadLine ();
    }
}

class FactorizeCompletion
{
    public void FactorizedResults (IAsyncResult ar)
    {
        // the IAsyncResult interface pointer ar can be casted
        // to an AsyncResult object
        AsyncResult result = (AsyncResult) ar;

        // the AsyncResult object has the AsyncDelegate property
        // (of type Object), which is the delegate object
        // on which the asynchronous call was invoked
        FactorizeHandler fd = (FactorizeHandler) result.AsyncDelegate;

        long factor1; 
        long factor2; 
        bool ret = fd.EndInvoke (out factor1, out factor2, ar);

        // output results
        if (ret)
        {
            Console.WriteLine ("found factors {0} and {1}.",
                factor1, factor2);
        }
    }
}
#endif
