using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Threading;
using System.Drawing;

class FlyingShapesForm : Form
{
    // threading utilities
    private Thread tr;
    private Thread tt;
    private bool stopped;

    // drawing utilities
    private Pen pr;
    private Pen pt;
    private Random random;

    // coordinates of shapes
    private int rx, ry;
    private int tx, ty;

    public FlyingShapesForm ()
    {
        // setup form
        this.ClientSize = new Size (350, 100);
        this.Text = "Flying Shapes";
        this.BackColor = Color.White;

        // initialize random number generator
        this.random = new Random ();

        // reset coordinates
        this.rx = this.ry = -1;
        this.tx = this.ty = -1;
    }

    protected override void OnLoad (EventArgs e)
    {
        // enable registered clients to receive 'Load' event
        base.OnLoad (e);

        this.pr = new Pen (Color.Red, 8);
        this.pt = new Pen (Color.Green, 7);

        // create a couple of threads
        ThreadStart ts1 = new ThreadStart (this.FlyingRectangle);
        this.tr = new Thread (ts1);
        this.tr.Name = "Rectangles";
        this.tr.Priority = ThreadPriority.BelowNormal;

        ThreadStart ts2 = new ThreadStart (this.FlyingTriangle);
        this.tt = new Thread (ts2);
        this.tt.Name = "Triangles";
        this.tt.Priority = ThreadPriority.BelowNormal;

        // enable thread procedures running
        this.tr.Start ();
        this.tt.Start ();
        this.stopped = false;
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        // enable registered clients to receive 'Closing' event
        base.OnClosing (e);

        // terminate threads
        this.stopped = true;
        this.tr.Join ();
        this.tt.Join ();

        // release resources
        this.pr.Dispose ();
        this.pt.Dispose ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' event
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // drawing rectangle
        if (this.rx != -1)
        {
            g.DrawRectangle (this.pr, this.rx, this.ry, 20, 20);
        }

        // drawing triangle
        if (this.tx != -1)
        {
            Point p1 = new Point (this.tx, this.ty);
            Point p2 = new Point (this.tx + 20, this.ty);
            Point p3 = new Point (this.tx + 10, this.ty - 15);
            Point[] points = { p1, p2, p3 };
            g.DrawPolygon (this.pt, points);
        }
    }

    private void FlyingRectangle ()
    {
        while (! this.stopped)
        {
            // calculate next shape position
            this.rx = this.random.Next (this.ClientRectangle.Width - 25);
            this.ry = this.random.Next (this.ClientRectangle.Height - 25);

            this.Invalidate ();

            // suspend thread
            Thread.Sleep (500);
        }
    }

    private void FlyingTriangle ()
    {
        while (! this.stopped)
        {
            // calculate next shape position
            this.tx = this.random.Next (this.ClientRectangle.Width - 25);
            this.ty = this.random.Next (this.ClientRectangle.Height - 25);

            this.Invalidate ();

            // suspend thread
            Thread.Sleep (1300);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        FlyingShapesForm f = new FlyingShapesForm ();
        Application.Run (f);
    }
}
