#define THREAD_01
// #define THREAD_02
// #define THREAD_03
// #define THREAD_04
// #define THREAD_05
// #define THREAD_06
// #define THREAD_07
// #define THREAD_08

using System;
using System.Threading;

#if THREAD_01
class Demo
{
    public static void Main ()
    {
        // need an instance of class ThreadDemo
        ThreadDemo obj = new ThreadDemo ();

        // creating a ThreadStart delegate
        ThreadStart ts = new ThreadStart (obj.Procedure);

        // creating a Thread instance
        Thread t = new Thread (ts);

        // start the thread
        t.Start ();
    }
}

class ThreadDemo
{
    public void Procedure ()
    {
        for (int i = 0; i < 5; i ++)
        {
            Console.WriteLine ("Hello Multithreading");
        }
    }
}
#endif

#if THREAD_02
class ThreadDemo
{
    private String s;

    public ThreadDemo (String s)
    {
        this.s = s;
    }

    public void Procedure ()
    {
        for (int i = 0; i < 5; i ++)
        {
            Console.WriteLine (this.s);

            // allow other waiting threads to execute
            Thread.Sleep (0);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        ThreadDemo obj1 = new ThreadDemo ("Hello ...");
        ThreadDemo obj2 = new ThreadDemo ("... World");

        ThreadStart ts1 = new ThreadStart (obj1.Procedure);
        ThreadStart ts2 = new ThreadStart (obj2.Procedure);

        Thread t1 = new Thread (ts1);
        Thread t2 = new Thread (ts2);

        t1.Start ();
        t2.Start ();
    }
}
#endif

#if THREAD_03
class ThreadDemo
{
    private static String s = "Hello Multithreading";

    public static void Procedure ()
    {
        for (int i = 0; i < 5; i ++)
        {
            Console.WriteLine (s);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        // creating a ThreadStart delegate
        ThreadStart ts = new ThreadStart (ThreadDemo.Procedure);

        // creating a Thread instance
        Thread t = new Thread (ts);

        // start the thread
        t.Start ();
    }
}
#endif

#if THREAD_04
class ThreadDemo
{
    public void Procedure ()
    {
        Thread t = Thread.CurrentThread;
        String s = t.Name;

        for (int i = 0; i < 5; i ++)
        {
            Console.WriteLine (s);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        ThreadDemo obj1 = new ThreadDemo ();
        ThreadDemo obj2 = new ThreadDemo ();

        ThreadStart ts1 = new ThreadStart (obj1.Procedure);
        ThreadStart ts2 = new ThreadStart (obj2.Procedure);

        // creating two named Thread instances
        Thread t1 = new Thread (ts1);
        Thread t2 = new Thread (ts2);
        t1.Name = "First Worker Thread";
        t2.Name = "Another Worker Thread";

        t1.Start ();
        t2.Start ();
    }
}
#endif

#if THREAD_05
class ThreadDemo
{
    public static void Procedure ()
    {
        Console.WriteLine ("entering 'Procedure'");
    }
}

class Demo
{
    public static void Main ()
    {
        ThreadStart ts = new ThreadStart (ThreadDemo.Procedure);
        Thread t = new Thread (ts);
        t.Start ();
        t.Join ();

        Console.WriteLine ("leaving 'Main'");
    }
}
#endif

#if THREAD_06
class ThreadDemo
{
    public static void Procedure ()
    {
        while (true)
            Console.WriteLine ("Hello Procedure");
    }
}

class Demo
{
    public static void Main ()
    {
        // create secondary thread
        ThreadStart ts = new ThreadStart (ThreadDemo.Procedure);
        Thread t = new Thread (ts);

        // mark secondary thread as background thread
        t.IsBackground = true;

        // start secondary thread
        t.Start ();

        Console.WriteLine ("leaving 'Main'");
    }
}
#endif

#if THREAD_07
class ThreadDemo
{
    public static void Procedure ()
    {
        Console.WriteLine ("entering secondary thread");
        try
        {
            while (true)
            {
                Console.WriteLine ("... running and ...");

                Thread.Sleep (1000);
            }
        }
        catch (ThreadAbortException e)
        {
            Console.WriteLine (e.Message);
        }
        finally
        {
            Console.WriteLine ("leaving secondary thread");
        }
    }
}

class Demo
{
    public static void Main ()
    {
        Console.WriteLine ("entering primary thread");

        // create secondary thread
        ThreadStart ts = new ThreadStart (ThreadDemo.Procedure);
        Thread t = new Thread (ts);

        // start secondary thread
        t.Start ();

        // give secondary thread 3 seconds to live ...
        Thread.Sleep (3000);

        // destroy secondary thread
        t.Abort ();
        t.Join ();

        Console.WriteLine ("leaving primary thread");
    }
}
#endif

#if THREAD_08
class ThreadDemo
{
    public static bool isRunning = true;

    public static void Procedure ()
    {
        Console.WriteLine ("entering secondary thread");
        while (isRunning)
        {
            Console.WriteLine ("... running and ...");
            Thread.Sleep (1000);
        }
        Console.WriteLine ("leaving secondary thread");
    }
}

class Demo
{
    public static void Main ()
    {
        Console.WriteLine ("entering primary thread");

        // create secondary thread
        ThreadStart ts = new ThreadStart (ThreadDemo.Procedure);
        Thread t = new Thread (ts);

        // start secondary thread
        t.Start ();

        // give secondary thread 3 seconds to live ...
        Thread.Sleep (3000);

        // destroy secondary thread
        ThreadDemo.isRunning = false;
        t.Join ();

        Console.WriteLine ("leaving primary thread");
    }
}
#endif

