using System;
using System.Threading;

class ThreadDemo
{
    public void ThreadProcedure ()
    {
        int hashCode = Thread.CurrentThread.GetHashCode ();

        try
        {
            Console.WriteLine ("{0}: entering thread procedure", hashCode);

            ThreadState state = Thread.CurrentThread.ThreadState;
            Console.WriteLine ("{0}: {1}", hashCode, state);

            // determine whether thread is running
            if ((state & (ThreadState.Unstarted | ThreadState.Stopped)) == 0)
            {
                Console.WriteLine ("{0}: Running", hashCode);
            }

            // switch into 'WaitSleepJoin' state
            try
            {
                Thread.Sleep (5000);
            }
            catch (ThreadInterruptedException)
            {
                state = Thread.CurrentThread.ThreadState;
                Console.WriteLine ("{0}: {1}", hashCode, state);
            }

            // wait for 'ThreadAbortException'
            Thread.Sleep (5000);
        }
        catch (ThreadAbortException)
        {
            ThreadState state = Thread.CurrentThread.ThreadState;
            Console.WriteLine ("{0}: {1}", hashCode, state);
        }
        finally 
        {
            Console.WriteLine ("{0}: leaving thread procedure", hashCode);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        int hashCode = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("{0}: entering main thread", hashCode);

        ThreadDemo obj = new ThreadDemo ();
        ThreadStart ts = new ThreadStart (obj.ThreadProcedure);
        Thread t = new Thread (ts);
        t.IsBackground = true;
        Console.WriteLine ("{0}: {1}", hashCode, t.ThreadState);

        // start the thread
        t.Start ();

        Thread.Sleep (1000);
        Console.WriteLine ("{0}: {1}", hashCode, t.ThreadState);

        // wakeup thread
        t.Interrupt ();
        Console.WriteLine ("{0}: {1}", hashCode, t.ThreadState);
        Thread.Sleep (1000);

        // terminate thread
        t.Abort ();
        Console.WriteLine ("{0}: {1}", hashCode, t.ThreadState);

        // wait for end of thread
        t.Join ();
        Console.WriteLine ("{0}: {1}", hashCode, t.ThreadState);
        Console.WriteLine ("{0}: leaving main thread", hashCode);
    }
}