#define VARIANT_01
// #define VARIANT_02

/**

1. Variante: Wie im Buch beschrieben

2. Variante: Legt nicht pro OnPaint-Aufruf ein neues Bitmap an,
             sondern kopiert aus dem ersten Bitmap mit DrawImage
             die entsprechenden Anteile um.
             Vorteil: Zwei (teure) Clone-Aufrufe knnen entfallen
             Nachteil: Kann Dispose zur Laufzeit nicht demonstrieren ;-)))
**/

using System;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;


#if VARIANT_01
class Ticker : Form
{
    private Bitmap map;
    private int width;
    private int height;
    private int position;

    private Thread t;
    private bool isActive;

    // default c'tor
    public Ticker ()
    {
        // create font
        FontFamily family = new FontFamily ("Lucida Console");
        Font f = new Font (
            family,            // font family
            48,                // size of the new font object
            FontStyle.Regular  // font style to be applied
        );

        // calculate width of string to draw (in pixel)
        String s = ".NET is Cool ;-)";
        Graphics g = this.CreateGraphics ();
        SizeF stringSize = g.MeasureString (s, f);
        this.width = (int) Math.Round (stringSize.Width);

        // setup form
        this.Text = "Simple Text Ticker";
        this.height = 150;
        this.ClientSize = new Size (this.width - 1, this.height);
        this.FormBorderStyle = FormBorderStyle.Fixed3D;

        // create original bitmap
        this.map = new Bitmap (this.width, this.height);
        Graphics og = Graphics.FromImage (this.map);
        og.FillRectangle (Brushes.Black, 0, 0, this.width, this.height);
        PointF pointF = new PointF (0, 40);
        og.DrawString (s, f, Brushes.Yellow, pointF);

        // release font specific resources
        family.Dispose ();
        f.Dispose ();

        // reset image position
        this.position = 0;

        // create thread
        ThreadStart ts = new ThreadStart (this.Trigger);
        this.t = new Thread (ts);
        this.t.Priority = ThreadPriority.Lowest;
        this.isActive = true;
        t.Start ();
    }

    private void Trigger ()
    {
        while (this.isActive)
        {
            this.Invalidate ();
            Thread.Sleep (20);
        }
    }
    
    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        // adjust picture position
        this.position ++;
        if (this.position == this.width - 1)
            this.position = 1;
        
        // create temporary bitmap for left part
        Rectangle r1 = new Rectangle (
            this.width - this.position,      // x
            0,                               // y
            this.position,                   // width
            this.height                      // height
        );
        Bitmap map1 = this.map.Clone (r1, PixelFormat.DontCare);

        // create temporary bitmap for right part
        Rectangle r2 = new Rectangle (
            0,                               // x
            0,                               // y
            this.width - this.position - 1,  // width
            this.height                      // height
        );
        Bitmap map2 = this.map.Clone (r2, PixelFormat.DontCare);

        // draw images
        Graphics g = e.Graphics;
        g.DrawImage (map1, 0, 0);
        g.DrawImage (map2, this.position, 0);

        // release temporary resources
//        map1.Dispose ();
//        map2.Dispose ();
    }

    protected override void OnPaintBackground (PaintEventArgs pe)
    {
        // do *not* handle any erase background request from windows
        // (to avoid flickering!)
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        // take care of registered delegates
        base.OnClosing (e);

        // terminate thread
        this.isActive = false;
        this.t.Join ();

        // release bitmap
        this.map.Dispose ();
    }
}
#endif


#if VARIANT_02
class Ticker : Form
{
    private Bitmap mapOrig;
    private Bitmap mapCopy;
    private int width;
    private int height;
    private int position;

    private Thread t;
    private bool isActive;

    // default c'tor
    public Ticker ()
    {
        // create font
        FontFamily family = new FontFamily ("Lucida Console");
        Font f = new Font (
            family,            // font family
            48,                // size of the new font object
            FontStyle.Regular  // font style to be applied
            );

        // calculate width of string to draw (in pixel)
        String s = ".NET is Cool ;-)";
        Graphics g = this.CreateGraphics ();
        SizeF stringSize = g.MeasureString (s, f);
        this.width = (int) Math.Round (stringSize.Width);

        // setup form
        this.Text = "Simple Text Ticker";
        this.height = 150;
        this.ClientSize = new Size (this.width - 1, this.height);
        this.FormBorderStyle = FormBorderStyle.Fixed3D;

        // create original bitmap
        this.mapOrig = new Bitmap (this.width, this.height);
        Graphics og = Graphics.FromImage (this.mapOrig);
        og.FillRectangle (Brushes.Black, 0, 0, this.width, this.height);
        PointF pointF = new PointF (0, 40);
        og.DrawString (s, f, Brushes.Yellow, pointF);

        // create second bitmap
        this.mapCopy = new Bitmap (this.width, this.height);

        // release font specific resources
        family.Dispose ();
        f.Dispose ();

        // reset image position
        this.position = this.width;

        // create thread
        ThreadStart ts = new ThreadStart (this.Trigger);
        this.t = new Thread (ts);
        this.t.Priority = ThreadPriority.Lowest;
        this.isActive = true;
        t.Start ();
    }

    private void Trigger ()
    {
        while (this.isActive)
        {
            this.Invalidate ();
            Thread.Sleep (20);
        }
    }
    
    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        // adjust picture position
        this.position --;
        if (this.position == 0)
            this.position = this.width;

        // create temporary bitmap for left part
        Rectangle r1 = new Rectangle (
            this.position,                   // x
            0,                               // y
            this.width,                      // width
            this.height                      // height
        );

        // create temporary bitmap for right part
        Rectangle r2 = new Rectangle (
            0,                               // x
            0,                               // y
            this.position,                   // width
            this.height                      // height
        );

        Graphics og = Graphics.FromImage (this.mapCopy);
        og.DrawImage (this.mapOrig, 0, 0, r1, GraphicsUnit.Pixel);
        og.DrawImage (this.mapOrig, this.width - this.position, 0, r2, GraphicsUnit.Pixel);

        // draw image
        Graphics g = e.Graphics;
        g.DrawImage (this.mapCopy, 0, 0);
    }

    protected override void OnPaintBackground (PaintEventArgs pe)
    {
        // do *not* handle any erase background request from windows
        // (to avoid flickering!)
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        // take care of registered delegates
        base.OnClosing (e);

        // terminate thread
        this.isActive = false;
        this.t.Join ();

        // release bitmap
        this.mapOrig.Dispose ();
        this.mapCopy.Dispose ();
    }
}
#endif
