#define THREAD_01
// #define THREAD_02
// #define THREAD_03

// #define THREAD_04
// #define THREAD_05

/***


Varianten 1 - 3: Wie im Buch beschrieben

Varianten 4 - 5: Wie oben, mit folgenden Unterschieden:

a) Push und Pop werden bei unzulssigem Stapel-Zustand nicht verlassen,
   sondern es wird mit while gewartet --> busy polling
   
b) Es ist eine Klasse StressManager ergnzt worden, um einen Absturz
   von Push/Pop fr den unsynchronisierten Fall demonstrieren zu knnen:
   
   Ist leider nur sehr selten reproduzierbar ....

***/

using System;
using System.Threading;

#if THREAD_01
class Stack
{
    private int[] buf;
    private int index;

    public Stack ()
    {
        this.buf = new int[5];
        this.index = 0;
    }

    public void Push (int n)
    {
        if (this.IsFull ())
            return;

        this.buf[this.index] = n;
        this.index ++;
    }

    public int Pop () 
    {
        if (this.IsEmpty ())
            return -1;

        this.index --;
        Thread.Sleep (0); // relinquish control: enable other thread(s) to run
        return this.buf[this.index];
    }

    public bool IsEmpty () 
    {
        return this.index == 0;
    }

    public bool IsFull () 

    {
        return this.index == this.buf.Length;
    }
}
#endif

#if THREAD_02
class Stack
{
    private int[] buf;
    private int index;

    public Stack ()
    {
        this.buf = new int[5];
        this.index = 0;
    }

    public void Push (int n)
    {
        Monitor.Enter (this);
        if (this.IsFull ())
        {
            Monitor.Exit (this);
            return;
        }

        this.buf[this.index] = n;
        this.index ++;
        Monitor.Exit (this);
    }

    public int Pop () 
    {
        Monitor.Enter (this);
        if (this.IsEmpty ())
        {
            Monitor.Exit (this);
            return -1;
        }

        this.index --;
        int val = this.buf[index];
        Monitor.Exit (this);
        return val;
    }

    public bool IsEmpty () 
    {
        return this.index == 0;
    }

    public bool IsFull () 

    {
        return this.index == this.buf.Length;
    }
}
#endif

#if THREAD_03
class Stack
{
    private int[] buf;
    private int index;

    public Stack ()
    {
        this.buf = new int[5];
        this.index = 0;
    }

    public void Push (int n)
    {  
        lock (this)
        {
            if (this.IsFull ())
            return;

            this.buf[this.index] = n;
            this.index ++;
        }
    }

    public int Pop () 
    {
        lock (this)
        {
            if (this.IsEmpty ())
                return -1;

            this.index --;
            return this.buf[this.index];
        }            
    }

    public bool IsEmpty () 
    {
        return this.index == 0;
    }

    public bool IsFull () 
    {
        return this.index == this.buf.Length;
    }
}
#endif

#if THREAD_01 || THREAD_02 || THREAD_03

class StressClient
{
    private Stack s;

    public StressClient (Stack s)
    {
        this.s = s;
    }

    public void FillStack ()
    {
        Console.WriteLine ("> FillStack");

        for (int i = 0; i < Int32.MaxValue; i ++)
            this.s.Push (i);

        Console.WriteLine ("< FillStack");
    }

    public void EmptyStack ()
    {
        Console.WriteLine ("> EmptyStack");

        for (int i = 0; i < Int32.MaxValue; i ++)
            this.s.Pop ();

        Console.WriteLine ("< EmptyStack");
    }
}

class Test
{
    public static void Main ()
    {
        StressTest ();
    }

    private static void StressTest ()
    {
        Stack s = new Stack ();
        StressClient c = new StressClient (s);

        Thread t0 = new Thread (new ThreadStart (c.EmptyStack));
        Thread t1 = new Thread (new ThreadStart (c.FillStack));

        int ticks = Environment.TickCount;

        t0.Start ();
        t1.Start ();
        t0.Join ();
        t1.Join ();

        ticks = Environment.TickCount - ticks;

        Console.WriteLine ("Ticks: {0}", ticks);

        Console.WriteLine ("< Main");
    }
}
#endif


#if THREAD_04
class Stack
{
    private long[] buf;
    private long index;

    public Stack ()
    {
        this.buf = new long[1];
        this.index = 0;
    }

    public void Push (long n)
    {
        while (this.IsFull ())
            ;

        this.buf[this.index] = n;
        this.index ++;
    }

    public long Pop () 
    {
        while (this.IsEmpty ())
            ;

        this.index --;
        // Thread.Sleep (0); // relinquish control: enable other thread(s) to run
        return this.buf[this.index];
    }

    public bool IsEmpty () 
    {
        return this.index == 0;
    }

    public bool IsFull () 
    {
        return this.index == this.buf.Length;
    }
}
#endif

#if THREAD_05
class Stack
{
    private long[] buf;
    private long index;

    public Stack ()
    {
        this.buf = new long[1];
        this.index = 0;
    }

    public void Push (long n)
    {
        Monitor.Enter (this);

        while (this.IsFull ())
        {
            Monitor.Exit (this);
            Thread.Sleep (0);
            Monitor.Enter (this);
        }

        this.buf[this.index] = n;
        this.index ++;

        Monitor.Exit (this);
    }

    public long Pop () 
    {
        Monitor.Enter (this);

        while (this.IsEmpty ())
        {
            Monitor.Exit (this);
            Thread.Sleep (0);
            Monitor.Enter (this);
        }

        this.index --;
        Thread.Sleep (0); // relinquish control: enable other thread(s) to run
        long val = this.buf[this.index];
    
        Monitor.Exit (this);
        return val;

    }

    public bool IsEmpty () 
    {
        return this.index == 0;
    }

    public bool IsFull () 
    {
        return this.index == this.buf.Length;
    }
}
#endif


#if THREAD_04 || THREAD_05
class StressClient
{
    private Stack s;

    public StressClient (Stack s)
    {
        this.s = s;
    }

    public void FillStack ()
    {
        Console.WriteLine ("> FillStack");

        for (int i = 0; i < 100000; i ++)
            this.s.Push (i);

        Console.WriteLine ("< FillStack");
    }

    public void EmptyStack ()
    {
        Console.WriteLine ("> EmptyStack");

        for (int i = 0; i < 100000; i ++)
            this.s.Pop ();

        Console.WriteLine ("< EmptyStack");
    }
}

class StressManager
{
    private Thread[] workersPush;
    private Thread[] workersPop;

    public StressManager (ThreadStart tsPush, ThreadStart tsPop, int numThreads)
    {
        this.workersPush = new Thread[numThreads];
        this.workersPop = new Thread[numThreads];

        for (int i = 0; i < this.workersPush.Length; i ++)
        {
            this.workersPush[i] = new Thread (tsPush);
            this.workersPush[i].Priority = ThreadPriority.Lowest;
        }
        for (int i = 0; i < this.workersPop.Length; i ++)
        {
            this.workersPop[i] = new Thread (tsPop);
            this.workersPop[i].Priority = ThreadPriority.Lowest;
        }
    }

    public void BeginStress ()
    {
        for (int i = 0; i < this.workersPush.Length; i ++)
        {
            this.workersPush[i].Start ();
            this.workersPop[i].Start ();
        }
    }

    public void EndStress ()
    {
        for (int i = 0; i < this.workersPush.Length; i ++)
        {
            this.workersPush[i].Join ();
            this.workersPop[i].Join ();
        }
    }
}

class Test
{
    public static void Main ()
    {
        StressTest ();
    }

    private static void StressTest ()
    {
        Console.WriteLine ("Press Enter ...");
        Console.ReadLine ();

        Stack s = new Stack ();
        StressClient c = new StressClient (s);

        ThreadStart ts1 = new ThreadStart (c.EmptyStack);
        ThreadStart ts2 = new ThreadStart (c.FillStack);

        StressManager manager = new StressManager (ts1, ts2, 10);

        int ticks = Environment.TickCount;

        try
        {
            manager.BeginStress ();
        }
        catch (Exception e)
        {
            Console.WriteLine ("Got exception {0}", e.Message);
            return;
        }

        manager.EndStress ();
        ticks = Environment.TickCount - ticks;

        Console.WriteLine ("Ticks: {0}", ticks);
        Console.WriteLine ("< Main");
    }
}
#endif
