#define VARIANT_01
// #define VARIANT_02

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

#if VARIANT_01
class IllegalWindowsFormsThreading : Form
{
    private Graphics g;

    public IllegalWindowsFormsThreading ()
    {
        // setup form
        this.ClientSize = new Size (400, 80);
        this.Text = "Illegal multithreaded Windows Form";
        this.FormBorderStyle = FormBorderStyle.Fixed3D;

        // setup button
        Button b = new Button ();
        b.Location = new Point (10, 10);
        b.Text = "Press me";
        b.Click += new EventHandler (this.Button_Click);
        this.Controls.Add (b);

        // retrieve graphics object for this control
        this.g = this.CreateGraphics ();
    }

    private void Stress ()
    {
        Color c = Color.FromName (Thread.CurrentThread.Name);
        Rectangle rect = new Rectangle (new Point (0, 0), this.ClientSize);
        Brush brush = new SolidBrush (c);
        for (int i = 0; i < 1000; i ++)
        {
            try
            {
                this.g.FillRectangle (brush, rect);
            }
            catch (Exception e)
            {
                Console.WriteLine ("Error: {0}: {1}", 
                    e.GetType ().ToString (), e.Message);
            }
        }
        brush.Dispose ();
    }
    
    private void Button_Click (Object sender, EventArgs e)
    {
        // create two threads
        ThreadStart ts = new ThreadStart (this.Stress);
        Thread t1 = new Thread (ts);
        Thread t2 = new Thread (ts);
        t1.Name = "Black";
        t2.Name = "White";

        // kick threads into running mode
        t1.Start ();
        t2.Start ();
    }
}

class Demo
{
    public static void Main () 
    {
        IllegalWindowsFormsThreading f = new IllegalWindowsFormsThreading ();
        Application.Run (f);
    }
}
#endif

#if VARIANT_02
delegate void FillRectangleHandler (Brush b, Rectangle r);

class LegalWindowsFormsThreading : Form
{
    private Graphics g;
    private FillRectangleHandler d;

    public LegalWindowsFormsThreading ()
    {
        // setup form
        this.ClientSize = new Size (400, 80);
        this.Text = "Legal multithreaded Windows Form";
        this.FormBorderStyle = FormBorderStyle.Fixed3D;

        // setup button
        Button b = new Button ();
        b.Location = new Point (10, 10);
        b.Text = "Press me";
        b.Click += new EventHandler (this.Button_Click);
        this.Controls.Add (b);

        // retrieve graphics object for this control
        this.g = this.CreateGraphics ();

        // create delegate object
        this.d = new FillRectangleHandler (this.g.FillRectangle);
    }

    private void Stress ()
    {
        Color c = Color.FromName (Thread.CurrentThread.Name);
        Rectangle rect = new Rectangle (new Point (0, 0), this.ClientSize);
        Brush brush = new SolidBrush (c);

        Object[] args = { brush, rect};

        for (int i = 0; i < 1000; i ++)
        {
            try
            {
                this.Invoke (this.d, args);
            }
            catch (Exception e)
            {
                Console.WriteLine ("Error: {0}: {1}", 
                    e.GetType ().ToString (), e.Message);
            }
        }
        brush.Dispose ();
    }
    
    private void Button_Click (Object sender, EventArgs e)
    {
        // create two threads
        ThreadStart ts = new ThreadStart (this.Stress);
        Thread t1 = new Thread (ts);
        Thread t2 = new Thread (ts);
        t1.Name = "Black";
        t2.Name = "White";

        // kick threads into running mode
        t1.Start ();
        t2.Start ();
    }
}

class Demo
{
    public static void Main () 
    {
        LegalWindowsFormsThreading f = new LegalWindowsFormsThreading ();
        Application.Run (f);
    }
}
#endif
