#define VARIANTE_01
// #define VARIANTE_02
// #define VARIANTE_03

using System;
using System.Threading;

#if VARIANTE_01
class AlarmDevice
{
    public void SetAlarm (int n)
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        // store alarm

        // release exclusive lock on this object
        Monitor.Exit (this);
    }

    public int GetAlarm ()
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        // retrieve alarm

        // return last-added alarm
        int alarm = 0;

        // release exclusive lock on this object
        Monitor.Exit (this);

        return alarm;
    }
}

class Demo
{
    public static void Main ()
    {
        AlarmDevice dev = new AlarmDevice ();
    }
}
#endif

#if VARIANTE_02
class AlarmDevice
{
    private int alarm;
    private bool available;

    public AlarmDevice ()
    {
        this.alarm = -1;
        this.available = false;
    }

    public int GetAlarm ()
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        while (! this.available)
        {
            // preparing some verbose output
            //Thread t = Thread.CurrentThread;
            //String s = t.Name;
            //Console.WriteLine ("No alarm available, thread {0} is going to wait ...", s);

            // currently no alarm available
            Monitor.Wait (this);
        }

        // return last-added alarm
        int alarm = this.alarm;
        this.available = false;

        // notifiy any threads in the condition queue
        // of a change in the locked object's state
        Monitor.Pulse (this);
        
        // release exclusive lock on this object
        Monitor.Exit (this);

        return alarm;
    }
    
    public void SetAlarm (int n)
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        while (this.available)
        {
            // last alarm has NOT been read
            Monitor.Wait (this);
        }

        // enter current alarm
        this.alarm = n;
        this.available = true;

        // notifiy any threads in the condition queue
        // of a change in the locked object's state
        Monitor.Pulse (this);

        // release exclusive lock on this object
        Monitor.Exit (this);
    }
}

class AlarmProducer
{
    private AlarmDevice device;

    public AlarmProducer (AlarmDevice device)
    {
        this.device = device;
    }

    public void ProcuceAlarms ()
    {
        for (int i = 0; i < 10; i ++)
        {
            this.device.SetAlarm (i);
            Thread.Sleep (500);
        }
    }
}

class AlarmConsumer
{
    private AlarmDevice device;

    public AlarmConsumer (AlarmDevice device)
    {
        this.device = device;
    }

    public void ConsumeAlarms ()
    {
        for (int i = 0; i < 10; i ++)
        {
            int alarm = this.device.GetAlarm ();
            Console.WriteLine ("got alarm {0}", alarm);
            Thread.Sleep (100);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        AlarmDevice device = new AlarmDevice ();

        AlarmProducer producer = new AlarmProducer (device);
        AlarmConsumer consumer = new AlarmConsumer (device);

        ThreadStart ts1 = new ThreadStart (producer.ProcuceAlarms);
        ThreadStart ts2 = new ThreadStart (consumer.ConsumeAlarms);

        Thread t0 = new Thread (ts1);
        Thread t1 = new Thread (ts2);

        t0.Start ();
        t1.Start ();

        t0.Join ();
        t1.Join ();

        Console.WriteLine ("end-of-main");
    }
}

#endif

#if VARIANTE_03
class AlarmDevice
{
    private int alarm;
    private bool available;

    public AlarmDevice ()
    {
        this.alarm = -1;
        this.available = false;
    }

    public int GetAlarm ()
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        while (! this.available)
        {
            // preparing some verbose output
            //Thread t = Thread.CurrentThread;
            //String s = t.Name;
            //Console.WriteLine ("No alarm available, thread {0} is going to wait ...", s);

            // currently no alarm available
            Monitor.Wait (this);
        }

        // return last-added alarm
        int alarm = this.alarm;
        this.available = false;

        // notifiy any threads in the condition queue
        // of a change in the locked object's state
        Monitor.PulseAll (this);
        
        // release exclusive lock on this object
        Monitor.Exit (this);

        return alarm;
    }

    public void SetAlarm (int n)
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        while (this.available)
        {
            // last alarm has NOT been read
            Monitor.Wait (this);
        }

        // enter current alarm
        this.alarm = n;
        this.available = true;

        // notifiy any threads in the condition queue
        // of a change in the locked object's state
        Monitor.PulseAll (this);

        // release exclusive lock on this object
        Monitor.Exit (this);
    }
}

class AlarmProducer
{
    private AlarmDevice device;
    private int start;

    public AlarmProducer (AlarmDevice device, int start)
    {
        this.device = device;
        this.start = start;
    }

    public void ProcuceAlarms ()
    {
        for (int i = this.start; i < this.start + 100; i ++)
        {
            this.device.SetAlarm (i);
        }
    }
}

class AlarmConsumer
{
    private AlarmDevice device;

    public AlarmConsumer (AlarmDevice device)
    {
        this.device = device;
    }

    public void ConsumeAlarms ()
    {
        for (int i = 0; i < 200; i ++)
        {
            int alarm = this.device.GetAlarm ();
            Console.WriteLine ("got alarm {0}", alarm);
//            Thread.Sleep (1000);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        AlarmDevice device = new AlarmDevice ();

        AlarmProducer p1 = new AlarmProducer (device, 0);
        AlarmProducer p2 = new AlarmProducer (device, 100);
        AlarmConsumer c1 = new AlarmConsumer (device);

        ThreadStart ts1 = new ThreadStart (p1.ProcuceAlarms);
        ThreadStart ts2 = new ThreadStart (p2.ProcuceAlarms);
        ThreadStart ts3 = new ThreadStart (c1.ConsumeAlarms);

        Thread t0 = new Thread (ts1);
        Thread t1 = new Thread (ts2);
        Thread t2 = new Thread (ts3);

        t0.Start ();
        t1.Start ();
        t2.Start ();

        t0.Join ();
        t1.Join ();
        t2.Join ();

        Console.WriteLine ("end-of-main");
    }
}

#endif
