using System;
using System.Threading;

class Demo
{
    public static void Main() 
    {
        // create an instance of class Stack
        Stack stack = new Stack ();

        // need a producer and a consumer
        Consumer consumer = new Consumer (stack);
        Producer producer = new Producer (stack);

        // creating two ThreadStart delegates
        ThreadStart ts1 = new ThreadStart (consumer.Run);
        ThreadStart ts2 = new ThreadStart (producer.Run);

        // creating two Thread instances
        Thread t1 = new Thread (ts1);
        Thread t2 = new Thread (ts2);

        // just for debugging purposes: set thread names
        t1.Name = "Consumer";
        t2.Name = "Producer";

        // now kick threads into the running mode ...
        t1.Start ();
        t2.Start ();

        // block main thread until threads have terminated
        t1.Join ();
        t2.Join ();
        Console.WriteLine ("Good bye Main");
    }
}

class Producer
{
    private Stack stack;

    public Producer (Stack stack)
    {
        this.stack = stack;
    }

    public void Run ()
    {
        Console.WriteLine ("> Producer");

        Random rand = new Random ();

        for (int i = 0; i < 10; i ++)
        {
            // push a random number onto the stack
            int n = rand.Next (10);
            Console.WriteLine ("Producer: pushes {0}", n);
            stack.Push (n);

            // suspend producer for a short period
            Thread.Sleep (2000);
        }

        Console.WriteLine ("< Producer");
    }
}

class Consumer
{
    private Stack stack;

    public Consumer (Stack stack)
    {
        this.stack = stack;
    }

    public void Run ()
    {
        Console.WriteLine ("> Consumer");

        for (int i = 0; i < 10; i ++)
        {
            Console.WriteLine ("Consumer: before pop request");
            int n = stack.Pop ();
            Console.WriteLine ("Consumer: popped {0}", n);

            // suspend consumer for a short period
            Thread.Sleep (200);
        }

        Console.WriteLine ("< Consumer");
    }
}