using System;
using System.Text;
using System.Threading;
using System.Collections;

class Stack
{
    private int[] buf;
    private int index;

    public Stack ()
    {
        buf = new int[10];
        index = 0;
    }

    public void Push (int n)
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        // if stack is full, release the monitor
        while (IsFull ())
        {
            // preparing some verbose output
            Thread t = Thread.CurrentThread;
            String s = t.Name;
            Console.WriteLine ("Stack is full, thread {0} is going to wait ...", s);
            
            Monitor.Wait (this);
        }

        // stack is non-full now, add element
        buf[index] = n;
        index ++;

        // notifiy all threads in the condition queue
        Monitor.PulseAll (this);

        // release exclusive lock on this object
        Monitor.Exit (this);
    }

    public int Pop () 
    {
        // acquire an exclusive lock on this object
        Monitor.Enter (this);

        // if stack is empty, release the monitor
        while (IsEmpty ())
        {
            // preparing some verbose output
            Thread t = Thread.CurrentThread;
            String s = t.Name;
            Console.WriteLine ("Stack is empty, thread {0} is going to wait ...", s);
            
            Monitor.Wait (this);
        }

        // stack is non-empty now, remove last element
        index --;
        int val = buf[index];

        // notifiy all threads in the condition queue
        Monitor.PulseAll (this);

        // release exclusive lock on this object
        Monitor.Exit (this);

        return val;
    }

    public int Pop_BusyPolling () 
    {
        int val = 0;
        bool isEmpty;

        do
        {
            Monitor.Enter (this);
            isEmpty = IsEmpty ();
            if (! isEmpty)
            {
                index --;
                val = buf[index];
            }
            Monitor.Exit (this);
        }
        while (isEmpty);

        return val;
    }

    public bool IsEmpty () 
    {
        return index == 0;
    }

    public bool IsFull () 

    {
        return index == buf.Length;
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder ("[");
        int count = buf.Length;
        for (int i = 0; i < count; i++)
        {
            int n = buf [i];
            sb.Append (n);
            if (i < count - 1)
                sb.Append (",");
        }
        sb.Append ("]");

        return sb.ToString ();
    }
}
