using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

class BubbleSortController : Panel
{
    private Button bStart;
    private Button bStop;
    private Button bShuffle;
    private Button bReverse;
    private ComboBox cbBarHeight;
    private ComboBox cbElements;
    private ComboBox cbInterval;
    private CheckBox cbMonitorEnabled;

    private BubbleSortView view;

    public BubbleSortController (BubbleSortView view)
    {
        this.view = view;

        // setup container
        this.BorderStyle = BorderStyle.Fixed3D;
        this.ClientSize = new Size (380, 36);

        // setup buttons
        this.bStart = new Button ();
        this.bStart.Text = "Start";
        this.bStart.Click += new EventHandler (this.OnClick);
        this.bStart.Width = 60;
        this.bStart.Location = new Point (10, 6);

        this.bStop = new Button ();
        this.bStop.Text = "Stop";
        this.bStop.Click += new EventHandler (this.OnClick);
        this.bStop.Width = 60;
        this.bStop.Location = new Point (80, 6);

        this.bShuffle = new Button ();
        this.bShuffle.Text = "Shuffle";
        this.bShuffle.Click += new EventHandler (this.OnClick);
        this.bShuffle.Width = 60;
        this.bShuffle.Location = new Point (150, 6);
        
        this.bReverse = new Button ();
        this.bReverse.Text = "Reverse";
        this.bReverse.Click += new EventHandler (this.OnClick);
        this.bReverse.Width = 60;
        this.bReverse.Location = new Point (220, 6);
        
        // setup label
        Label l1 = new Label ();
        l1.Text = "Bar Height:";
        l1.TextAlign = ContentAlignment.MiddleRight;
        l1.Width = 62;
        l1.Location = new Point (290, 6);

        // setup bar height combo box
        this.cbBarHeight = new ComboBox ();
        this.cbBarHeight.Items.Add (6);
        this.cbBarHeight.Items.Add (8);
        this.cbBarHeight.Items.Add (10);
        this.cbBarHeight.Items.Add (12);
        this.cbBarHeight.Items.Add (15);
        this.cbBarHeight.SelectedIndex = 0;
        this.cbBarHeight.SelectedIndexChanged +=
            new EventHandler (this.OnSelectedIndexChanged);
        this.cbBarHeight.Width = 40;
        this.cbBarHeight.Location = new Point (360, 6);

        // setup label
        Label l2 = new Label ();
        l2.Text = "Elements:";
        l2.TextAlign = ContentAlignment.MiddleRight;
        l2.Width = 60;
        l2.Location = new Point (410, 6);
        
        // setup elements combo box
        this.cbElements = new ComboBox ();
        this.cbElements.Items.Add (20);
        this.cbElements.Items.Add (30);
        this.cbElements.Items.Add (40);
        this.cbElements.Items.Add (50);
        this.cbElements.SelectedIndex = 1;
        this.cbElements.SelectedIndexChanged +=
            new EventHandler (this.OnSelectedIndexChanged);
        this.cbElements.Width = 40;
        this.cbElements.Location = new Point (480, 6);

        // setup label
        Label l3 = new Label ();
        l3.Text = "Interval:";
        l3.TextAlign = ContentAlignment.MiddleRight;
        l3.Width = 50;
        l3.Location = new Point (520, 6);
        
        // setup elements combo box
        this.cbInterval = new ComboBox ();
        this.cbInterval.Items.Add (0);
        this.cbInterval.Items.Add (100);
        this.cbInterval.Items.Add (500);
        this.cbInterval.Items.Add (1000);
        this.cbInterval.Items.Add (2000);
        this.cbInterval.SelectedIndex = 0;
        this.cbInterval.SelectedIndexChanged +=
            new EventHandler (this.OnSelectedIndexChanged);
        this.cbInterval.Width = 60;
        this.cbInterval.Location = new Point (580, 6);

        // setup monitor test combo box
        this.cbMonitorEnabled = new CheckBox ();
        this.cbMonitorEnabled.Text = "Monitor Enabled";
        this.cbMonitorEnabled.Checked = true;
        this.cbMonitorEnabled.CheckedChanged +=
            new EventHandler (OnCheckedChanged);
        this.cbMonitorEnabled.Width = 130;
        this.cbMonitorEnabled.Location = new Point (670, 6);

        // add controls to container
        this.Controls.Add (this.bStart);
        this.Controls.Add (this.bStop);
        this.Controls.Add (this.bShuffle);
        this.Controls.Add (this.bReverse);
        this.Controls.Add (l1);
        this.Controls.Add (this.cbBarHeight);
        this.Controls.Add (l2);
        this.Controls.Add (this.cbElements);
        this.Controls.Add (l3);
        this.Controls.Add (this.cbInterval);
        this.Controls.Add (this.cbMonitorEnabled);
    }

    private void OnClick (Object sender, EventArgs e)
    {
        if (sender == this.bStart)
        {
            this.view.Start ();
        }
        else if (sender == this.bStop)
        {
            this.view.Stop ();
        }
        else if (sender == this.bShuffle)
        {
            this.view.Shuffle ();
        }
        else if (sender == this.bReverse)
        {
            this.view.Reverse ();
        }
    }

    private void OnSelectedIndexChanged (Object sender, EventArgs e)
    {
        if (sender == this.cbBarHeight)
        {
            this.view.BarHeight = (int) this.cbBarHeight.SelectedItem;
        }
        else if (sender == this.cbElements)
        {
            this.view.Elements = (int) this.cbElements.SelectedItem;
        }
        else if (sender == this.cbInterval)
        {
            this.view.Interval = (int) this.cbInterval.SelectedItem;
        }
    }

    private void OnCheckedChanged (Object sender, EventArgs e)
    {
        this.view.MonitorEnabled = this.cbMonitorEnabled.Checked;
    }
}
