using System;
using System.Text;
using System.IO;
using System.Collections;

class CrossReference
{
    private StreamReader sr;
    private FileStream fs;

    private ArrayList tokens;
    private int count;

    private static String[] keywords =
    {
        // intentionally incomplete list of C# keywords
        "if", "else", "continue", "break", "while", "for", "return",
        "class", "new", "this", "null",
        "static", "public", "private",
        "void", "bool", "char", "int", "float", "double",
        "true", "false"
    };

    public void OpenFile (String name)
    {
        // open existing data file
        this.fs = new FileStream (name, FileMode.Open);

        // create stream reader for the specified stream
        this.sr = new StreamReader (this.fs);

        this.tokens = new ArrayList ();
        this.count = 0;
    }

    public void Parse ()
    {
        StringBuilder token = new StringBuilder ();
        while (true)
        {
            // read next line
            String line = this.sr.ReadLine ();
            if (line == null)
                return;

            this.count ++;
            for (int i = 0; i < line.Length; i ++)
            {
                // skip strings
                char c = line[i];
                if (c == '\"')
                {
                    // take care of escaped double quote character
                    // inside string literal, e.g. "\""
                    char lastch = c;
                    for (i ++; i < line.Length; i ++)
                    {
                        if (line[i] == '\"' && lastch != '\\')
                            break;
                        lastch = line[i];
                    }
                    continue;
                }

                // skip single line comments
                if (c == '/')
                {
                    i ++;
                    if (i < line.Length && line[i] == '/')
                        break;  // drop rest of line
                }

                // skip non initial characters of an identifier
                if (! Char.IsLetter (c))
                    continue;

                // parse identifier
                token.Append (c);
                for (i ++; i < line.Length; i ++)
                {
                    if (! Char.IsLetterOrDigit (line[i]))
                        break;
                    else
                        token.Append (line[i]); 
                }

                // drop keywords
                if (this.IsKeyword (token.ToString ()))
                {
                    token.Remove (0, token.Length);
                    continue;
                }

                // identifier found
                this.EnterToken (token.ToString (), this.count);
                token.Remove (0, token.Length);
            }
        }
    }

    public void CloseFile ()
    {
        // close reader and stream
        this.sr.Close ();
        this.fs.Close ();
    }

    public void Print ()
    {
        for (int i = 0; i < this.tokens.Count; i ++)
        {
            Token token = (Token) this.tokens[i];
            Console.Write ("{0,16}: ", token.Name);
            token.PrintLines ();
            Console.WriteLine ();
        }
    }

    private void EnterToken (String name, int line)
    {
        // token already exists in the list
        for (int i = 0; i < this.tokens.Count; i ++)
        {
            Token tok = (Token) this.tokens[i];
            if (tok.Name.Equals (name))
            {
                tok.EnterLine (line);
                return;
            }
        }

        // create new token
        Token token = new Token (name);
        token.EnterLine (line);

        // find alphabetic position to insert token
        int index = 0;
        while (index < this.tokens.Count)
        {
            Token tok = (Token) this.tokens[index];
            if (String.Compare (tok.Name, name) > 0)
                break;
            else
                index ++;
        }
        this.tokens.Insert (index, token);
    }

    private bool IsKeyword (String token)
    {
        for (int i = 0; i < keywords.Length; i ++)
            if (keywords[i].Equals (token))
                return true;

        return false;
    }
}
