using System;
using System.Text;
using System.Threading;
using System.IO;
using System.Net;
using System.Net.Sockets;

class HighLevelEchoServer
{
    public static void Main ()
    {
        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // create a listener for connections from TCP network clients
        TcpListener server = new TcpListener (ep);

        // start listening for client requests
        server.Start ();

        // enter the listening loop
        while (true)
        {
            // perform a blocking call to accept requests
            Console.WriteLine ("waiting for connection request ...");
            TcpClient client = server.AcceptTcpClient ();

            // serve a single client, using another thread context
            ClientConnection conn = new ClientConnection (client);
            ThreadStart ts = new ThreadStart (conn.ServeSingleClient);
            Thread t = new Thread (ts);
            t.Start ();
        }

        // shutdown and end this connection
        server.Stop ();
    }
}

class ClientConnection
{
    private TcpClient client;

    public ClientConnection (TcpClient client)
    {
        this.client = client;
    }

    public void ServeSingleClient ()
    {
        // get a stream object for reading and writing
        NetworkStream stream = client.GetStream();

        // create comfortable string input and output objects
        StreamReader reader = new StreamReader (stream, Encoding.ASCII);
        StreamWriter writer = new StreamWriter (stream, Encoding.ASCII);

        // flush data to the underlying stream
        // after every call to StreamWriter.WriteLine
        writer.AutoFlush = true;

        while (true)
        {
            // read single line from client
            String line = reader.ReadLine ();
            if (line.Trim ().ToLower ().Equals ("bye"))
                break;

            // copy tiny header in front of client's message
            line = String.Format ("EchoServer replies: {0}", line);
            Console.WriteLine (line);

            // send message to the client
            writer.WriteLine (line);
        }

        // shutdown and end this connection
        reader.Close ();
        writer.Close ();
        stream.Close ();
        client.Close ();
    }
}
