using System;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Globalization;

class Demo
{
    public static void Main ()
    {
        LowLevelDaytimeServer.HandleClients ();
    }
}

class LowLevelDaytimeServer
{
    public static void HandleClients ()
    {
        // initialize a new instance of the Socket class
        Socket socket = new Socket (
            AddressFamily.InterNetwork,
            SocketType.Stream,
            ProtocolType.Tcp
        );

        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // associate the socket with this local endpoint
        socket.Bind (ep);

        // cause socket to listen for incoming connection attempts
        socket.Listen (20);

        // extract next pending connection request from queue
        while (true)
        {
            Console.WriteLine ("waiting for connection request ...");
            Socket socket1 = socket.Accept ();

            // serve a single client, using another thread context
            ClientConnection conn = new ClientConnection (socket1);
            ThreadStart ts = new ThreadStart (conn.ServeSingleClient);
            Thread t = new Thread (ts);
            t.Start ();
        }
    }
}

class ClientConnection
{
    private Socket socket;

    public ClientConnection (Socket socket)
    {
        this.socket = socket;
    }

    public void ServeSingleClient ()
    {
        try
        {
            while (true)
            {
                // retrieve current local date and time on this computer
                DateTime dt = DateTime.Now;

                // send 'daytime' message to client
                String s = dt.ToString("U", DateTimeFormatInfo.InvariantInfo);
                byte[] msg = Encoding.ASCII.GetBytes (s + "\r\n");
                this.socket.Send (msg);

                // suspend client specific thread for a second
                Thread.Sleep (1000);
            }
        }
        catch (SocketException)
        {
            Console.WriteLine ("current connection closed.");
        }
        finally
        {
            // disable sends and receives on this Socket
            this.socket.Shutdown (SocketShutdown.Both);

            // close Socket connection, release all associated resources
            this.socket.Close ();
        }
    }
}
