using System;
using System.Threading;
using System.Text;
using System.Net;
using System.Net.Sockets;

class Demo
{
    public static void Main ()
    {
        LowLevelEchoServer.HandleConversations ();
    }
}

class LowLevelEchoServer
{
    public static void HandleConversations ()
    {
        // initialize a new instance of the Socket class
        Socket socket = new Socket (
            AddressFamily.InterNetwork,
            SocketType.Stream,
            ProtocolType.Tcp
        );

        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // associate the socket with this local endpoint
        socket.Bind (ep);

        // cause socket to listen for incoming connection attempts
        socket.Listen (20);

        // extract next pending connection request from queue
        while (true)
        {
            Console.WriteLine ("waiting for connection request ...");
            Socket socket1 = socket.Accept ();

            // serve a single client, using another thread context
            ClientConnection conn = new ClientConnection (socket1);
            ThreadStart ts = new ThreadStart (conn.ServeSingleClient);
            Thread t = new Thread (ts);
            t.Start ();
        }
    }
}

class ClientConnection
{
    private Socket socket;

    public ClientConnection (Socket socket)
    {
        this.socket = socket;
    }

    public void ServeSingleClient ()
    {
        while (true)
        {
            // read single line from client
            String line = ReadSingleLine ();
            Console.WriteLine ("found line: {0}", line);

            // check for end of conversation
            if (line.Trim ().ToLower ().Equals ("bye"))
                break;

            // copy tiny header in front of client's message
            line = String.Format ("EchoServer replies: {0}", line);
            byte[] msg = Encoding.ASCII.GetBytes (line);

            // send response message to client
            this.socket.Send (msg);
        }

        // disable sends and receives on this Socket
        this.socket.Shutdown (SocketShutdown.Both);

        // close Socket connection, release all associated resources
        this.socket.Close ();
    }

    private String ReadSingleLine ()
    {
        // read single line from client
        String line = "";
        byte[] bytes = new byte[1];
        while (this.socket.Receive (bytes) > 0)
        {
            // decode a byte array into an array of characters
            char[] chars = Encoding.ASCII.GetChars (bytes);

            // drop carriage return characters
            if (chars[0] == '\n')
                continue;

            // line feed character found, line is complete
            if (chars[0] == '\r')
                break;

            line = line + chars[0];
        }

        return line;
    }
}
