// #define VARIANTE01
#define VARIANTE02

#if VARIANTE01
using System;
using System.Text;
using System.Net;
using System.Net.Sockets;

class LowLevelHelloWorldServer
{
    public static void Main ()
    {
        // initialize a new instance of the Socket class
        Socket s = new Socket (
            AddressFamily.InterNetwork,
            SocketType.Stream,
            ProtocolType.Tcp
        );

        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // associate the socket with this local endpoint
        s.Bind (ep);

        // cause socket to listen for incoming connection attempts
        s.Listen (20);

        // extract next pending connection request from queue
        while (true)
        {
            Console.WriteLine ("waiting for connection request ...");
            Socket s1 = s.Accept ();

            // send message to client
            byte[] msg = Encoding.ASCII.GetBytes ("Hello World ...");
            s1.Send (msg);

            // disable sends and receives on this Socket
            s1.Shutdown (SocketShutdown.Both);

            // close Socket connection, release all associated resources
            s1.Close ();
        }

        // shutdown and end this connection
        s.Close ();
    }
}
#endif

#if VARIANTE02
using System;
using System.Threading;
using System.Text;
using System.Net;
using System.Net.Sockets;

class Demo
{
    public static void Main ()
    {
        LowLevelHelloWorldServer server =
            new LowLevelHelloWorldServer ();

        server.Start ();
        Console.WriteLine ("press any key to shutdown server ...");
        Console.ReadLine ();
        server.Shutdown ();
    }
}

class LowLevelHelloWorldServer
{
    private Socket s;
    private Thread t;

    public void Start ()
    {
        // initialize a new instance of the Socket class
        this.s = new Socket (
            AddressFamily.InterNetwork,
            SocketType.Stream,
            ProtocolType.Tcp
        );

        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // associate the socket with this local endpoint
        this.s.Bind (ep);
        
        // cause socket to listen for incoming connection attempts
        this.s.Listen (20);

        // create worker thread to handle client requests
        ThreadStart ts = new ThreadStart (this.ServeClients);
        this.t = new Thread (ts);
        this.t.Start ();
    }

    public void Shutdown ()
    {
        this.s.Close ();
        this.t.Join ();
    }

    public void ServeClients  ()
    {        
        // extract next pending connection request from queue
        while (true)
        {
            Console.WriteLine ("waiting for connection request ...");
            Socket s1;
            try
            {
                s1 = this.s.Accept ();
            }
            catch (SocketException)
            {
                break;
            }

            // send message to client
            byte[] msg = Encoding.ASCII.GetBytes ("Hello World ...");
            s1.Send (msg);

            // disable sends and receives on this Socket
            s1.Shutdown (SocketShutdown.Both);

            // close Socket connection, release all associated resources
            s1.Close ();
        }
    }
}
#endif

