using System;
using System.Threading;
using System.Collections;

class ChatMessenger
{
    private ChatSessionManager manager;
    private ArrayList messages;
    private Thread t;
    private bool isAlive;

    public ChatMessenger (ChatSessionManager manager)
    {
        this.manager = manager;
        this.messages = new ArrayList ();
    }

    public void Start ()
    {
        ThreadStart ts = new ThreadStart (this.DistributeMessages);
        this.t = new Thread (ts);
        this.isAlive = true;
        this.t.Start ();
    }

    public void Stop ()
    {
        Monitor.Enter (this);
        this.isAlive = false;
        Monitor.Pulse (this);
        Monitor.Exit (this);
        this.t.Join ();
    }

    public void EnterMessage (String s)
    {
        Monitor.Enter (this);
            
        // add message
        this.messages.Add (s);

        // resume message distribution
        Monitor.Pulse (this);
        Monitor.Exit (this);
    }

    private void DistributeMessages ()
    {
        // send incoming messages to all chat clients
        while (this.isAlive)
        {
            String s = this.RemoveMessage ();
            if (s == null)
                break;

            this.manager.DistributeMessage (s);
        }
    }
    
    private String RemoveMessage ()
    {
        Monitor.Enter (this);

        // queue is empty
        while (this.messages.Count == 0)
        {
            if (! this.isAlive)
            {
                // terminate message distribution
                Monitor.Exit (this);
                return null;
            }

            // suspend message distribution
            Monitor.Wait (this);
        }

        // remove entry
        String msg = (String) this.messages[0];
        this.messages.RemoveAt (0);

        Monitor.Exit (this);
        return msg;
    }
}
