using System;
using System.IO;
using System.Net.Sockets;
using System.Threading;
using System.Text;

class ChatSession
{
    private ChatSessionManager manager;
    private ChatMessenger messenger;
    private TcpClient client;
    private StreamReader reader;
    private StreamWriter writer;

    public ChatSession (
        ChatSessionManager manager,
        ChatMessenger messenger,
        TcpClient client)
    {
        this.manager = manager;
        this.messenger = messenger;
        this.client = client;
    }

    public void Start ()
    {
        ThreadStart ts = new ThreadStart (this.ServeSingleSession);
        Thread t = new Thread (ts);
        t.Start ();
    }

    private void ServeSingleSession ()
    {
        // create comfortable input and output streams
        NetworkStream stream = client.GetStream();
        this.reader = new StreamReader (stream, Encoding.ASCII);
        this.writer = new StreamWriter (stream, Encoding.ASCII);
        this.writer.AutoFlush = true;

        while (true)
        {
            String line;
            try
            {
                // wait for a message from the client
                line = this.reader.ReadLine ();
                Console.WriteLine ("char server received {0}", line);

                // handle end of conversation
                if (line == null)
                    break;
            }
            catch (IOException)
            {
                // corresponding socket has been closed
                break;
            } 

            this.messenger.EnterMessage (line);
        }

        // shutdown and remove session
        this.Shutdown ();
        this.manager.RemoveSession (this);
    }
  
    public void WriteMessage (String s)
    {
        this.writer.WriteLine (s);
    }

    public void Shutdown ()
    {
        // shutdown this connection
        this.reader.Close ();
        this.writer.Close ();
        this.client.Close ();
    }      
}
