using System;
using System.Collections;
using System.Threading;
using System.Net;
using System.Net.Sockets;

class ChatSessionManager
{
    private TcpListener server;
    private ArrayList sessions;
    private ChatMessenger messenger;
    private Thread t;

    public ChatSessionManager ()
    {
        this.sessions = new ArrayList ();
    }

    public void Start ()
    {
        ThreadStart ts = new ThreadStart (this.ServeSessions);
        this.t = new Thread (ts);
        this.t.Start ();
    }

    public void Stop ()
    {
        this.server.Stop ();
        this.Shutdown ();
        this.t.Join ();
    }

    public void AddSession (ChatSession session)
    {
        Monitor.Enter (this.sessions);
        this.sessions.Add (session);
        Monitor.Exit (this.sessions);
    }

    public void RemoveSession (ChatSession session)
    {
        Monitor.Enter (this.sessions);
        this.sessions.Remove (session);
        Monitor.Exit (this.sessions);
    }

    private void Shutdown ()
    {
        Monitor.Enter (this.sessions);
        for (int i = 0; i < this.sessions.Count; i ++)
        {
            ChatSession session = (ChatSession) this.sessions[i];
            session.Shutdown ();
        }

        this.sessions.Clear ();
        Monitor.Exit (this.sessions);
    }

    public void DistributeMessage (String s)
    {
        Monitor.Enter (this.sessions);
        for (int i = 0; i < this.sessions.Count; i ++)
        {
            ChatSession session = (ChatSession) this.sessions[i];
            Console.WriteLine ("sending reply to client at slot {0}", i);
            session.WriteMessage (s);
        }
        Monitor.Exit (this.sessions);
    }

    private void ServeSessions ()
    {
        // launch messenger thread
        this.messenger = new ChatMessenger (this);
        this.messenger.Start ();
        
        // create a listener for connections from TCP network clients
        IPAddress adr = IPAddress.Any;
        IPEndPoint ep = new IPEndPoint (adr, 4711);
        this.server = new TcpListener (ep);

        // start listening for client requests
        this.server.Start ();

        // enter the listening loop
        while (true)
        {
            // perform a blocking call to accept requests
            Console.WriteLine ("waiting for chat client request ...");
            TcpClient client;
            try
            {
                client = this.server.AcceptTcpClient ();
            }
            catch (SocketException)
            {
                Console.WriteLine ("server is closing ...");
                break;
            }

            ChatSession session =
                new ChatSession (this, this.messenger, client);

            this.AddSession (session);

            // serve this chat client (using another thread context)
            session.Start ();
        }

        // terminate notifications thread
        this.messenger.Stop ();
    }
}